
/** @file utils/list.nut List library functions. */

/**
 * An extended GSList class containing some utility functions for GSLists.
 */
 class List extends GSList {
 
	/**
	 * List constructor
	 * @param list optional GSList or List (this class) to copy.
	 */
	constructor(list = null) {
	
		::GSList.constructor();
		
		if (list != null) {
		
			// Validate.
			if(!(list instanceof GSList) && !(list instanceof List)) throw("List::Constructor: Argument has to be an GSList or List!");
			
			this.AddList(list);
		}
	}

/* public: */

/* private */
	
}

// Value/Item Functions.

/**
 * Valuator function for functions that change the game state and as such
 * are not allowed as function for GSList::Valuate.
 * @param valuator Function to be used as valuator
 * @param ... Additional argument to be passed to the valuator
 * @note max of 6 parameters.
 */
function List::DoValuate(valuator, ...) {

	// Validate.
	if(typeof(valuator) != "function") throw("List::DoValuate: argument has to be a function!");

	/* If values are changed while iterating over the list then some items
	 * might be skipped. Prevent this by caching all new values and applying
	 * them after iterating over all items. */
	local new_values = GSList();
	foreach(item, value in this) {
		local item_value = null;
		switch(vargc) {
            case 0:
                item_value = valuator(item);
			break;
            case 1:
                item_value = valuator(item, vargv[0]);
			break;
            case 2:
                item_value = valuator(item, vargv[0], vargv[1]);
			break;
            case 3:
                item_value = valuator(item, vargv[0], vargv[1], vargv[2]);
			break;
            case 4:
                item_value = valuator(item, vargv[0], vargv[1], vargv[2], vargv[3]);
			break;
            case 5:
                item_value = valuator(item, vargv[0], vargv[1], vargv[2], vargv[3], vargv[4]);
			break;
            case 6:
                item_value = valuator(item, vargv[0], vargv[1], vargv[2], vargv[3], vargv[4], vargv[5]);
			break;
            default:
                throw "Too many arguments to List::DoValuate Function!";
        }
		
		if(typeof item_value == "bool") item_value = item_value ? 1 : 0;
		new_values.AddItem(item, item_value);
	}
	
	this.Clear();
	this.AddList(new_values);
}

/*
 * Get a random item from the list, using the list values as weights. An item
 * with a higher weight value will be chosen with a higher frequency than an
 * item with a lower weight. 
 * @pre The weights must sum to the value of the sum * parameter (see ValueSum()).
 * @return A random item.
 */
function List::RandomItemByWeight() {
	local pivot = GSBase.RandRange(this.ValueSum());
	local item = this.Begin();
	local n = 0;
	
	while(this.HasNext()) {
		n += this.GetValue(item);
		
		if(n >= pivot) {
			break;
		} else {
			item = this.Next();
		}
	}

	return item;
}

/**
 * Set the value of all items to the item itself.
 * Eg, {item, item}
 */
function List::SetValueAsItem() {
	/* We should never change a value while iterating over the list so create
	 * a copy first so we can iterate over the copy. */
	local copy = GSList();
	copy.AddList(this);
	foreach(item, value in copy) this.SetValue(item, item);
}

/**
 * Reset the list so that values are the items.
 * @note Items entries will be lost if their values are the same,
 *	in this case it will be set as the last value in list.
 * Eg, {value, 0}
 */
function List::SetItemAsValue() {
	/* We should never change a value while iterating over the list so create
	 * a copy first so we can iterate over the copy. */
	local copy = GSList();
	copy.AddList(this);
	this.Clear();
	foreach(item, value in copy) this.AddItem(value, 0);
}

/*
 * Get the sum of all values in a the list.
 * @return An integer of the sum of all values.
 */
function List::ValueSum() {
	local sum = 0;
	for(local i = this.Begin(); this.HasNext(); i = this.Next()) {
		sum += list.GetValue(i);
	}
	return sum;
}

/**
 * Add items to list from array,
 * All the new items are added with {array[i], 0}.
 * @pre typeof add_array == "array"
 */
function List::AddArrayItems(add_array) {
	assert(typeof add_array == "array");
	foreach(item in add_array) {
		this.AddItem(item, 0);
	}
}

/**
 * Add items to list from table,
 * All the new items are added with {item, value}.
 * @pre typeof add_table == "table"
 * @pre All values are integer type.
 */
function List::AddTable(add_table) {
	assert(typeof add_table == "table");
	foreach(item, value in add_table) {
		assert(typeof value == "integer");
		this.AddItem(item, value);
	}
}

// Convert Functions.

/**
 * Get all of list items as an array.
 * @return An array with all items in it.
 */
function List::ItemsToArray() {
	local new_array = [];
	foreach(item, value in this) {
		new_array.append(item);
	}
	return new_array;
}

/**
 * Get all of list values as an array.
 * @return An array with all values in it.
 */
function List::ValuesToArray() {
	local new_array = [];
	foreach(item, value in this) {
		new_array.append(value);
	}
	return new_array;
}

/**
 *	Convert the list to a table.
 *	@return A table containing all {item 1, value 1}, {item 2, value 2}, etc pairs from the list.
 */
function List::ToTable() {
	local table = {};
	foreach(item, value in this) {
		table.rawset(item, value);
	}
	return table;
}

/**
 *	Convert the list to a human-readable string.
 *	@return A string containing all {item 1, value 1}, {item 2, value 2}, etc pairs from the list.
 */
function List::ToString() {
	local string = "";
	foreach(item, value in this) {
		if(string == "") string += "{" + item + ", " + value + " }"
		else string += ", {" + item + ", " + value + " }";
	}
	return string;
}

// Sort Functions.

/**
 * Sort by item, ascending.
 */
function List::SortItemAscending() {
	this.Sort(this.SORT_BY_ITEM, this.SORT_ASCENDING);
}

/**
 * Sort by item, descending.
 */
function List::SortItemDescending() {
	this.Sort(this.SORT_BY_ITEM, this.SORT_DESCENDING);
}

/**
 * Sort by value, ascending.
 */
function List::SortValueAscending() {
	this.Sort(this.SORT_BY_VALUE, this.SORT_ASCENDING);
}

/**
 * Sort by value, descending.
 */
function List::SortValueDescending() {
	this.Sort(this.SORT_BY_VALUE, this.SORT_DESCENDING);
}

// Preview Functions

/**
 * Pre-count items left if Removes all items with a value above start and below end.
 * @param start  the lower bound of the to be removed values (exclusive).
 * @param end  the upper bound of the to be removed valuens (exclusive).
 * @return count of left item.
 */
function List::CountIfRemoveBetweenValue(start, end) {
	local temp_list = GSList();
	temp_list.AddList(this);
	temp_list.RemoveBetweenValue(start, end);
	return temp_list.Count();
}

/**
 * Pre-count items left if Removes all items with a higher value than 'value'.
 * @param value the value above which all items are removed.
 * @return count of left item.
 */
function List::CountIfRemoveAboveValue(value) {
	local temp_list = GSList();
	temp_list.AddList(this);
	temp_list.RemoveAboveValue(value);
	return temp_list.Count();
}

/**
 * Pre-count items left if Removes all items with a lower value than 'value'.
 * @param value the value below which all items are removed.
 * @return count of left item.
 */
function List::CountIfRemoveBelowValue(value) {
	local temp_list = GSList();
	temp_list.AddList(this);
	temp_list.RemoveBelowValue(value);
	return temp_list.Count();
}

/**
 * Pre-count items left if Remove all items with this value.
 * @param value the value to remove.
 * @return count of left item.
 */
function List::CountIfRemoveValue(value) {
	local temp_list = GSList();
	temp_list.AddList(this);
	temp_list.RemoveValue(value);
	return temp_list.Count();
}

/**
 * Pre-count items left if Keep all items with this value.
 * @param value the value to keep.
 * @return count of left item.
 */
function List::CountIfKeepValue(value) {
	local temp_list = GSList();
	temp_list.AddList(this);
	temp_list.KeepValue(value);
	return temp_list.Count();
}

/**
 * Pre-count items left if Remove n items from top.
 * @param number the number of item to remove from top.
 * @return count of left item.
 */
function List::CountIfRemoveTop(number) {
	local temp_list = GSList();
	temp_list.AddList(this);
	temp_list.RemoveTop(number);
	return temp_list.Count();
}

/**
 * Pre-count items left if Remove n items from bottom.
 * @param number the number of item to remove from bottom.
 * @return count of left item.
 */
function List::CountIfRemoveBottom(number) {
	local temp_list = GSList();
	temp_list.AddList(this);
	temp_list.RemoveBottom(number);
	return temp_list.Count();
}

// Queue Functions

/**
 * Get the topmost item from the list
 * @pre !IsEmpty()
 * @return the topmost item of the list
 */
function List::Peek() {
	assert(!this.IsEmpty());
	return this.Begin();
}

/**
 * Remove the topmost item from the list and return it.
 * @pre !IsEmpty()
 * @return the topmost item of the list
 */
function List::Pop() {
	assert(!this.IsEmpty());
	local item = this.Begin();
	this.RemoveItem(item);
	return item;
}

/**
 * Add an item to the list.
 * Adds {item, 0}.
 */
function List::Push(item) {
	this.AddItem(item, 0);
}