
/** @file library/log.nut Log library functions. */

/*   This file is part of MapConstructor, which is a GameScript for OpenTTD
 *   Copyright (C) 2013  R2dical
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * A library class containing some utility functions for log messaging.
 */
class Log {
	
	constructor() {}
	
/* public: */
	
	// Log.Level
	static	MAIN = 1;
	static	SUB = 2;
	static	DETAIL = 3;
	
	// Log.Colour
	static	BLACK = 1;
	static	WHITE = 2;
	static	RED = 3;
	
/* private */

}

/**
 *	Display a normal (black) log msg if meets log level.
 *	@param text The text to display, is casted to string.
 *	@param level Optional Log.Level to display text.
 *	@pre GSController.GetSetting("log_level") returns a value.
 */
function Log::Info(text = "", level = Log.MAIN) {
	if(level <= GSController.GetSetting("log_level"))
		GSLog.Info("" + text);
}

/**
 *	Display a warning (white) log msg if meets log level.
 *	@param text The text to display, is casted to string.
 *	@param level Optional Log.Level to display text.
 *	@pre GSController.GetSetting("log_level") returns a value.
 */
function Log::Warning(text = "", level = Log.MAIN) {
	if(level <= GSController.GetSetting("log_level"))
		GSLog.Warning("" + text);
}

/**
 *	Display a error (red) log msg if meets log level.
 *	@param text The text to display, is casted to string.
 *	@param level Optional Log.Level to display text.
 *	@pre GSController.GetSetting("log_level") returns a value.
 */
function Log::Error(text = "", level = Log.MAIN) {
	if(level <= GSController.GetSetting("log_level")) {
		GSLog.Error(text);
	}
}

/**
 *	Display a special log msg iff debug on.
 *	@param text The text to display, is casted to string.
 *	@param level Optional Log.Colour to display text.
 *	@pre GSController.GetSetting("debug_level") returns a value.
 */
function Log::Debug(text = "", colour = Log.BLACK) {
	local debug_level = GSController.GetSetting("debug_level");
	if(debug_level ==  1 || debug_level == 3) {
		switch(colour) {
			case Log.BLACK: this.Info("" + text); break;
			case Log.WHITE: this.Warning("" + text); break;
			case Log.RED: this.Error("" + text); break;
			default: Log.Error("Invalid colour!");
		}
	}
}

/**
 *	Display a break (red) log msg if dev tools are active.
 *	@param text The text to display, is casted to string.
 */
function Log::Break(text = "") {
	GSController.Break("" + text);
}

/**
 *	Convert any type of object to a string with a recursive call.
 *	Suited for debug display.
 *	@param object The object to convert.
 *	@return A string with the expanded and explicit data types and values.
 */
function Log::ObjectToString(object) {

	local txt = null;

	switch (typeof(object)) {
		case "instance":
			if(object instanceof GSList) {
				txt = "(list) ";
				if(!object.IsEmpty()) {
					foreach(item, value in object)
						txt += "{" + item + " -> " + Log.ObjectToString(value) + "}, ";
					txt = txt.slice(0, txt.len()-2);
					return txt;
				} else txt += "(empty)";
			}
			txt = object.tostring();
		break;

		case "function":
			txt = object.tostring();
		break;
		
		case "array":
			txt = "(" + typeof(object) + ") ";
			if(object.len() > 0) {
				foreach(item, value in object)
					txt += "[" + item + "]: " + Log.ObjectToString(value) + ", ";
				txt = txt.slice(0, txt.len() - 2);
			} else txt += "[]";
		break;
		
		case "table":
			txt = "(" + typeof(object) + ") ";
			if(object.len() > 0) {
				foreach(item, value in object)
					txt += "{" + item + " -> " + Log.ObjectToString(value) + "}, ";
				txt = txt.slice(0, txt.len()-2);
			} else txt += "{}";
		break;

		case  "bool":
			txt = "(" + typeof(object) + ") ";
			txt += object;
		break;
		
		case  "null":
			txt = "(" + typeof(object) + ") ";
			txt += object;
		break;
		
		case "string":
			txt = "(" + typeof(object) + ") ";
			txt += object;
		break;
		
		case  "float":
			txt = "(" + typeof(object) + ") ";
			txt += object;
		break;
		
		case  "integer":
			txt = "(" + typeof(object) + ") ";
			txt += object;
		break;
		
	   default:
			txt = object.tostring();
	}
	return txt;
}
