/** @file main.nut Main loop and staring point for Game Script. */

/*   
 *   This file is part of MapConstructor, which is a GameScript for OpenTTD
 *   Copyright (C) 2013  R2dical
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// External libraries.
import("util.superlib", "SuperLib", 36); 				///< Import SuperLib

// Internal libraries.
require("require.nut");

// TODO
// Handle not buildable and check for prospect.

/**
 * Main class of the script.
 */
class MapConstructor extends GSController {
	
	constructor() {
	
		this.map_size_x = GSMap.GetMapSizeX();
		this.map_size_y = GSMap.GetMapSizeY();
		this.map_scale = (this.map_size_x / 256.0) * (this.map_size_y / 256.0);
		this.build_industries_limit = GSController.GetSetting("build_industries_limit");
		this.industry_type_list = GSIndustryTypeList();
		this.cargo_paxid = SuperLib.Helper.GetPAXCargo();
		this.load_performed = false;
		this.first_industry_build_performed = false;
		this.primary_performed = false;
		this.secondary_performed = false;
		this.tertiary_performed = false;
		this.special_performed = false;
		this.raw_industry_array = [];
		this.processor_industry_array = [];
		this.tertiary_industry_array = [];
		this.special_industry_array = [];
		this.special_industry_names = ["Bank", "Oil Rig", "Water Tower", "Lumber Mill"];
		this.abnormal_industry_names = ["Oil Refinery", "Farm", "Forest", "Water Supply", "Fruit Plantation", "Rubber Plantation"];
		this.density_industries_special_method = 5;
		this.industry_random_build_tries = min(100000, ((256 * 256 * 1) * this.map_scale).tointeger());
		this.industry_scattered_build_tries = min(100000, ((256 * 256 * 2) * this.map_scale).tointeger());
		this.industry_town_build_tries = min(100000, (1000 * GSController.GetSetting("town_industry_maximum_distance")));
		this.industry_cluster_table = {};
		this.industry_clustered_build_tries = min(100000, ((256 * 256 * 1) * this.map_scale).tointeger());
		this.industry_clustered_node_tries = min(100000, ((256 * 256 * 1) * this.map_scale).tointeger());
		this.town_metropolis_table = {};
		this.town_metropolis_node_tries = min(100000, ((256 * 256 * 1) * this.map_scale).tointeger());
		this.town_metropolis_build_tries = min(100000, ((256 * 256 * 1) * this.map_scale).tointeger());
		
		this.density_towns_total = GSController.GetSetting("density_towns_total");
		this.density_towns_minimim_proportion = 0.8;
		this.build_towns_limit = 1;
		this.build_towns_chance = 10;
		this.town_random_build_tries = ((256 * 256 * 1) * this.map_scale).tointeger();
		this.town_scattered_build_tries = ((256 * 256 * 1) * this.map_scale).tointeger();
		this.town_clustered_build_tries = min(100000, ((256 * 256 * 1) * this.map_scale).tointeger());
		this.town_clustered_node_tries = min(100000, ((256 * 256 * 1) * this.map_scale).tointeger());
		this.town_cluster_table = {}
	}
	
	map_size_x = null;
	map_size_y = null;
	map_scale = null;
	
	industry_type_list = null;									///< The GSIndustryTypeList() to be used for this game, set in constructor.
	cargo_paxid = null;											///< Passenger cargo ID.
	
	build_industries = null;									///< Global build industries check, set and checked in Initialize().
	build_towns = null;											///< Global build town check, set and checked in Initialize().
	build_industries_limit = null; 								///< Set from settings, in class constructor and each refresh (Initial is maximum industries per map,
																///< 	subsequent is maximum per refresh).
	build_towns_limit = null;									///< Set in constructor. Max towns to build per refresh.
	build_towns_chance = null;									///< Set in constructor. Chance out 100 to roll for building towns each refresh, which can still roll 0.
	
	load_performed = null;										///< If this is a loaded game or new game.
	first_industry_build_performed = null;						///< If Load() OR BuildIndustry() has run. A safety measure for unexpected save/load.
	first_town_build_performed = null;							///< If Load() OR BuildTown() has run. A safety measure for unexpected save/load.
	primary_performed = null;									// True if BuildIndustry() has run for primary industries.
	secondary_performed = null;									// True if BuildIndustry() has run for secondary industries.
	tertiary_performed = null;									// True if BuildIndustry() has run for tertiary industries.
	special_performed = null;									// True if BuildIndustry() has run for special industries.
	
	raw_industry_array = null;									///< Array of primary industry type ID's, set in Initialize().
	processor_industry_array = null;							///< Array of secondary industry type ID's, set in Initialize().
	tertiary_industry_array = null;								///< Array of tertiary industry type ID's, set in Initialize().
	special_industry_array = null;								///< Array of special industry type ID's, set in Initialize().
	special_industry_names = null;								///< Array of strings of special industry names, set in constructor.
	abnormal_industry_names = null;								///< Array of strings of abnormal industry names, set in constructor. DO NOT CHANGE ORDER!! INDEX USED FOR REFERENCE!!
	
	industry_random_build_tries = null;							///< The number of build tries available for IndustryRandomBuildMethod(). Set in constructor.
	industry_scattered_build_tries = null;						///< The number of build tries available for IndustryScatteredBuildMethod(). Set in constructor.
	industry_clustered_build_tries = null;						///< The number of build tries available for IndustryClusteredBuildMethod(). Set in constructor.
	industry_clustered_node_tries = null;						///< The number of tries for nodes available for IndustryClusteredBuildMethod(). Set in constructor.
	industry_town_build_tries = null;							///< The number of build tries available for IndustryTownBuildMethod(). Set in constructor.
	
	industry_cluster_table = null;								///< 2D Table which stores data of industry cluster nodes. {industry_type_id, {node_id, [tile_index, tile_index ...(max_per_cluster)]}}
	town_cluster_table = null;									///< Table which stores data of town cluster nodes. {node_id, [tile_index, tile_index ...(max_per_cluster)]}
	town_metropolis_table = null;								///< Table which stores data of metropolis nodes. {node_id, [tile_index, tile_index ...(max_per_cluster)]}
	
	town_random_build_tries = null;								///< The number of build tries available for TownRandomBuildMethod(). Set in constructor.
	town_scattered_build_tries = null;							///< The number of build tries available for TownScatteredBuildMethod(). Set in constructor.
	town_clustered_build_tries = null;							///< The number of build tries available for TownClusteredBuildMethod(). Set in constructor.
	town_clustered_node_tries = null;							///< The number of tries for nodes available for TownClusteredBuildMethod(). Set in constructor.
	town_metropolis_build_tries = null;							///< The number of build tries available for TownScatteredBuildMethod(). Set in constructor.
	town_metropolis_node_tries = null;							///< The number of tries for nodes available for TownMetropolisBuildMethod(). Set in constructor.
	
	// User variables
	density_towns_total = null;									///< Set from settings, in constructor. Total towns, integer always >= 1.
	density_towns_minimim_proportion = null;					///< Set in constructor. The minimum proportion of maximum to build initially. float < 1.
	
	density_industries_total = null;							///< Set from settings, in Initialize(). Total industries, integer always >= 1.
	density_industries_minimum = null;							///< Set from settings, in Initialize(). Min industry density %, float always < 1.
	density_industries_maximum = null;							///< Set from settings, in Initialize(). Max industry density %, float always > 1. Doesn't affect initial build.
	density_industries_raw = null;								///< Set from settings, in Initialize(). Primary industry proportion, float always < 1.
	density_industries_processor = null;						///< Set from settings, in Initialize(). Secondary industry proportion, float always < 1.
	density_industries_tertiary = null;							///< Set from settings, in Initialize(). Tertiary industry proportion, float always < 1.
	density_industries_special = null;							///< Set from settings, in Initialize(). Special industry proportion, float always < 1.
	density_industries_raw_method = null;						///< Set from settings, in Initialize(). (1 = "Town", 2 = "Clusters", 3 = "Scattered", 4 = "Random").
	density_industries_processor_method = null;					///< Set from settings, in Initialize(). (1 = "Town", 2 = "Clusters", 3 = "Scattered", 4 = "Random").
	density_industries_tertiary_method = null;					///< Set from settings, in Initialize(). (1 = "Town", 2 = "Clusters", 3 = "Scattered", 4 = "Random").
	density_industries_special_method = null;					///< Set in constructor, special case non-selectable. (5 = "Special");
	
}

/**
 *	Save all data we need to be able to resume later in a 'table' = {name = value, name = value, ...}
 *	@return A table containing all data that needs to be saved.
 *	@note This is called by OpenTTD, no need to call from within the GS.
 */
function MapConstructor::Save() {
	
	// Display msg.
	Log.Info("+==============================+", Log.MAIN);
	Log.Info("Saving, version " + ::current_version + "...", Log.MAIN);
	
	// Create the save data table
	local save_data = 	{
							save_raw_industry_array = raw_industry_array,
							save_processor_industry_array = processor_industry_array,
							save_tertiary_industry_array = tertiary_industry_array,
							save_special_industry_array = special_industry_array,
							save_industry_cluster_table = industry_cluster_table,
							save_town_cluster_table = town_cluster_table
						};
	
	return save_data;
}

/**
 *	Load the savegame data we get.
 *	@param version The version of this GS that was used to save the game. Provided by controller.
 *	@param data The data that was stored by Save(). Provided by controller.
 *	@note This is called by OpenTTD, no need to call from within the GS.
 */
function MapConstructor::Load(load_version, load_data) {

	// Display msg.
	Log.Info("+==============================+", Log.MAIN);
	Log.Info("Loading version " + load_version + "...", Log.MAIN);
	
	// Check version.
	if(load_version < ::current_version)
		Log.Warning("Loading older script version data!", Log.MAIN);
	else if(load_version > ::current_version)
		Log.Warning("Loading newer script version data!", Log.MAIN);
	
	// Load data.
	foreach(save_key, save_value in load_data) {
		
		// Switch for match.
		switch(save_key) {
			
			case "save_raw_industry_array" : this.raw_industry_array = save_value;
			case "save_processor_industry_array" : this.processor_industry_array = save_value;
			case "save_tertiary_industry_array" : this.tertiary_industry_array = save_value;
			case "save_special_industry_array" :  this.special_industry_array = save_value;
			case "save_industry_cluster_table" : this.industry_cluster_table = save_value;
			case "save_town_cluster_table" : this.town_cluster_table = save_value;
			default : Log.Error("Unknown save data!", Log.MAIN);
			
		}
	}
	
	// Update load status.
	this.load_performed = true;
	this.first_town_build_performed = true;
	this.first_industry_build_performed = true;
}

/**
 *	The starting function, containing the script loop.
 *	@note This is called by OpenTTD, no need to call from within the GS.
 */
function MapConstructor::Start() {

	// Welcome msg.
	Log.Info("Starting Industry Constructor version " + ::current_version + "!", Log.MAIN);
	Log.Info();

	// Check settings.
	if(!this.CheckSettings())
		return;
	
	// Initialize.
	if(!this.Initialize())
		return;
	
	// Analyse settings.
	if(!this.AnalyseSettings())
		return;
	
	// GS loop.
	local gs_continue = null;
	do { gs_continue = this.Main(); } while(gs_continue);
	
	// Exit msg.
	Log.Info();
	Log.Warning("The game script has ended, and will now crash.", Log.MAIN);
	Log.Info();
	return;
}

/**
 *	The main game script loop function.
 *	@return True if completed, false if GS must end.
 *	@note Assumes 1 day = 74 ticks.
 */
function MapConstructor::Main() {
	
	// Handle events.
	this.HandleEvents();
	
	// Build town function.
	if(this.build_towns)
		this.BuildTown();
	
	// Build industry function.
	if(this.build_industries)
		this.BuildIndustry();
	
	// Clear events.
	this.ClearEvents()
	
	// First sleep.
	this.Sleep(max(1, GSController.GetSetting("build_speed") * 2252));
	
	return true;
}

/**
 *	Check if all game settings are valid in the current openttd version.
 *	@return True if all settings are valid, false if one is not.
 */
function MapConstructor::CheckSettings() {

	// Display msg.
	Log.Info("+==============================+", Log.SUB);
	Log.Info("Checking settings...", Log.SUB);
	
	local setting_array =  ["economy.multiple_industry_per_town", 
							"economy.found_town", 
							"economy.town_layout", 
							"economy.larger_towns", 					// <0-255
							"economy.initial_city_size", 				// <1-10>
							"game_creation.oil_refinery_limit", 
							"game_creation.town_name"];

	local all_valid = true;
	foreach (setting in setting_array) {
	
		if(!GSGameSettings.IsValid(setting)) {
			Log.Error("MapConstructor::CheckSettings: Setting " + setting + " is invalid!", Log.MAIN);
			all_valid = false;
		}
	}
	
	return all_valid;
}

/**
 *	Analyse some settings for how they will affect the GS.
 *	@pre All used settings are validated already.
 */
function MapConstructor::AnalyseSettings() {
	
	// Display msg.
	Log.Info("+==============================+", Log.SUB);
	Log.Info("Analyzing settings...", Log.SUB);
	
	// Multi ind per town.
	if(GSGameSettings.GetValue("multiple_industry_per_town") == 0) {
		Log.Warning("> Multiple industries per town disabled, will slow down or prevent some build methods!", Log.MAIN);
		
		// Check for cluster industry methods (2).
		if((density_industries_raw_method == 2) || (density_industries_processor_method == 2) || (density_industries_tertiary_method == 2)) {
			this.WarningBox("You have chosen a Clustered build method but have Multiple Industries per Town disabled, it is highly unlikely this combination will succeed!");
		}
	}
		
	// Oil refinery distance.
	if(GSGameSettings.GetValue("oil_refinery_limit") < 20)
		Log.Warning("> Oil refinery distance from edge limit is quite low, this can cause trouble generating enough refineries!", Log.MAIN);
	
	return true;
}

/**
 *	The initialization function, validates and prepares script. Must finish and return true for GS to continue.
 *	@return True if completed and success, false if GS must end.
 *	@note This is called by OpenTTD, no need to call from within the GS.
 */
function MapConstructor::Initialize() {

	// Display msg.
	Log.Info("+==============================+", Log.MAIN);
	Log.Info("Initializing...", Log.MAIN);
	
	// Set build towns and industries.
	// - Towns.
	if(GSController.GetSetting("build_towns") == 1) {
	
		// - Check if already towns on map.
		if(GSTown.GetTownCount() > 1) {
			Log.Warning("> Set to build towns but there are already " + GSTown.GetTownCount() + 
				" town on map! Build towns will now be disabled.", Log.MAIN);
			this.build_towns = false;
		} else {
			Log.Info("> Set to generate towns.", Log.SUB);
			this.build_towns = true;
		}
	}
	// - Industries.
	if(GSController.GetSetting("build_industries") == 1) {
	
		// - Check if already industries on map.
		local map_ind_list = GSIndustryList();
		if(!map_ind_list.IsEmpty()) {
			Log.Warning("> Set to build industries but there are already " + map_ind_list.Count() + 
				" industries on map! Build industries will now be disabled.", Log.MAIN);
			this.build_industries = false;
		} else {
			Log.Info("> Set to generate industries.", Log.SUB)
			this.build_industries = true;
		}
	}	
	// - Check if the script has something to do.
	if(!this.build_industries && !this.build_towns) {
		Log.Error("> Set to not build industries or towns! Nothing to do!", Log.MAIN);
		return false;
	}
	
	// Map scale (compared to 256 x 256).
	Log.Info("> Map size scale is: " + this.map_scale, Log.SUB);
	
	// Check passenger cargo id.
	if(this.cargo_paxid == -1)
		return false;
	
	// If load has not happened, then this is a new game.
	if(!this.load_performed) {
	
		// Display info msg.
		Log.Info("> This is a new game, preparing...", Log.SUB);
		
		// Loop through ind list.
		foreach(industry_id, _ in this.industry_type_list) {
		
			// Check if is a special/abnormal industry.
			local is_special = false;
			local is_abnormal = false;
			
			// - Get current ID name.
			local industry_name = GSIndustryType.GetName(industry_id);
			
			// - Loop through special list.
			foreach(special_name in this.special_industry_names) {
				// - If current ID name is a special = special_industry_array.
				if(industry_name == special_name) {
					// - Assign true and end loop.
					is_special = true;
					break;
				}
			}
			
			// - Loop through abnormal list.
			foreach(abnormal_name in this.abnormal_industry_names) {
				// - If current ID name is a special = special_industry_array.
				if(industry_name == abnormal_name) {
					// - Assign true and end loop.
					is_abnormal = true;
					break;
				}
			}
			
			// If the current ID was special.
			if(is_special) {
			
				// Display info msg
				Log.Info("> Special Industry: " + industry_name + ".", Log.DETAIL);
				
				// Add industry id to raw list.
				this.special_industry_array.append(industry_id);
				
			// Else if current ID is a raw producer = raw_industry_array.
			} else if (GSIndustryType.IsRawIndustry(industry_id)) {
			
				// Display industry type name msg
				Log.Info("> Raw Industry: " + industry_name + ".", Log.DETAIL);
				
				// Add industry id to raw list.
				this.raw_industry_array.append(industry_id);
				
			// Else if current ID is a processor = processor_industry_array.
			} else if (GSIndustryType.IsProcessingIndustry(industry_id)) {
			
				// Display industry type name msg.
				Log.Info("> Processor Industry: " + industry_name + ".", Log.DETAIL);
				
				// Add industry id to processor list
				this.processor_industry_array.append(industry_id);
				
			// Else is an other industry = tertiary_industry_array.
			} else {
			
				// Display industry type name msg.
				Log.Info("> Tertiary Industry: " + industry_name + ".", Log.DETAIL);
				
				// Add industry id to other list.
				this.tertiary_industry_array.append(industry_id);
			}
			
			// Display is abnormal.
			if(is_abnormal)
				Log.Info("-> Abnormal Industry!", Log.DETAIL);
		}
	// Else not a new game
	} else
		// Display status msg
		Log.Info("> This is a loaded game, preparing...", Log.SUB);
		
	Log.Info("> Detected " + this.industry_type_list.Count() + " industry types.", Log.SUB);
	
	// Import settings
	// - Assign settings
	local raw_industry_proportion = GSController.GetSetting("density_industries_raw").tofloat();
		if(raw_industry_array.len() == 0) raw_industry_proportion = 0.0;
	local processor_industry_proportion = GSController.GetSetting("density_industries_processor").tofloat();
		if(processor_industry_array.len() == 0) processor_industry_proportion = 0.0;
	local tertiary_industry_proportion = GSController.GetSetting("density_industries_tertiary").tofloat();
		if(tertiary_industry_array.len() == 0) tertiary_industry_proportion = 0.0;
	local special_industry_proportion = GSController.GetSetting("density_industries_special").tofloat();
		if(special_industry_array.len() == 0) special_industry_proportion = 0.0;
	
	local total_industries_proportion = (raw_industry_proportion + processor_industry_proportion + tertiary_industry_proportion + special_industry_proportion).tofloat();
	
	// - Assign proportions.
	this.density_industries_total = max(1, (GSController.GetSetting("density_industries_total") * this.map_scale).tointeger());
	this.density_industries_minimum = GSController.GetSetting("density_industries_minimum") / 100.0;
	this.density_industries_maximum = GSController.GetSetting("density_industries_maximum") / 100.0;
	this.density_industries_raw = raw_industry_proportion / total_industries_proportion;
	this.density_industries_processor = processor_industry_proportion / total_industries_proportion;
	this.density_industries_tertiary = tertiary_industry_proportion / total_industries_proportion;
	this.density_industries_special = special_industry_proportion / total_industries_proportion;
	
	// - Assign methods.
	this.density_industries_raw_method = GSController.GetSetting("density_industries_raw_method");
	this.density_industries_processor_method = GSController.GetSetting("density_industries_processor_method");
	this.density_industries_tertiary_method = GSController.GetSetting("density_industries_tertiary_method");
	
	// - Display status msgs
	Log.Info("> Total industries assigned: " + this.density_industries_total, Log.SUB);
	Log.Info("> Min per industry assigned: " + this.density_industries_minimum, Log.SUB);
	Log.Info("> Max per industry assigned: " + this.density_industries_maximum, Log.SUB);
	Log.Info("> Primary industry proportion assigned: " + this.density_industries_raw, Log.SUB);
	Log.Info("> Secondary industry proportion assigned: " + this.density_industries_processor, Log.SUB);
	Log.Info("> Tertiary industry proportion assigned: " + this.density_industries_tertiary, Log.SUB);
	Log.Info("> Special industry proportion assigned: " + this.density_industries_special, Log.SUB);
	Log.Info("> Primary industry method assigned: " + density_industries_raw_method, Log.SUB);
	Log.Info("> Secondary industry method assigned: " + density_industries_processor_method, Log.SUB);
	Log.Info("> Tertiary industry method assigned: " + density_industries_tertiary_method, Log.SUB);
	
	// End	
	return true;
}

/**
 *	The function to build towns initially and each refresh period.
 *	Builds up to a random number of each industry, between min and max according to proportion.
 */
function MapConstructor::BuildTown() {

	// Display msg.
	Log.Info("+==============================+", Log.MAIN);
	Log.Info("Building towns...", Log.MAIN);
	
	// Build limits.
	local current_build_limit = 0;			// Build limit in this function
	
	// If first build has not happened, then this is a new game.
	if(!this.first_town_build_performed) {
		
		// Set limit as a random & below max, minus initial town.
		current_build_limit = (this.density_towns_total * this.map_scale * this.density_towns_minimim_proportion).tointeger() + GSBase.RandRange((this.density_towns_total * this.map_scale * (1 - this.density_towns_minimim_proportion)).tointeger()) - 1;
		
		// Display status msg.
		Log.Info("> This is a new game, town build limit is: " + current_build_limit + ".", Log.SUB);
		
		// Set first build
		this.first_town_build_performed = true;
		
	// Else build has happened, and we will build a few only.
	} else {
	
		// Set build limit to a random up to global.
		current_build_limit = GSBase.Chance(this.build_towns_chance, 100) ? GSBase.RandRange(this.build_towns_limit + 1) : 0;
		
		// Display status msg.
		Log.Info("> This is a continuing game, town build limit is: " + current_build_limit + " / " + this.build_towns_limit + " (Max).", Log.SUB);
	}
	
	// Check to continue.
	if(current_build_limit == 0)
		return;
	
	// Get settings.
	// - % town growth, either custom or random.
	local town_growth = null;
	if(GSController.GetSetting("economy_town_growth") > 0)
		town_growth = GSController.GetSetting("economy_town_growth") / 100.0;		
	// - Smallest allowed town size.
	local minimum_town_size = GSController.GetSetting("density_town_size");
	// - Town layout design to use.
	local town_layout = GSGameSettings.GetValue("economy.town_layout");
	// Maximum cities.
	local maximum_cities = GSGameSettings.GetValue("economy.larger_towns") == 0 ? 0 : (current_build_limit * (1.0 / GSGameSettings.GetValue("economy.larger_towns"))).tointeger();
	// Maximum towns.
	local maximum_towns = current_build_limit - maximum_cities;
	
	// Check if initial town is city then decrement appropriately.
	local town_list = GSTownList();
	if(GSTown.IsCity(town_list.Begin())) {
		// Display msg
		Log.Info(" ~First town is city.", Log.DETAIL);
		maximum_cities--;
		maximum_towns--;
	} else {
		// Display msg
		Log.Info(" ~First town is not city.", Log.DETAIL);
		maximum_towns--;
	}
	
	// Display msg.
	Log.Info("> Target building " + maximum_towns + " towns, and " + maximum_cities + " cities.", Log.SUB);
	
	// Loop variables.
	local town_size = null;
	local town_build_method = GSController.GetSetting("density_towns_method");
	local town_index = null;;
	local town_count = 0;
	local city_count = 0;
	
	// Loop while below current build limit.
	while((town_count + city_count) < current_build_limit) {
		
		// Set city.
		local is_city = false;
		//if((city_count < maximum_cities) && (GSBase.Chance(maximum_cities, current_build_limit))) {
		
		// - First always a city.
		if((town_count + city_count) == 0) {
			is_city = true;
			town_size = GSTown.TOWN_SIZE_LARGE;
			
		// Special city / town selector. Calculates so [city, town, town, town], [city, town, town, town], [city, etc...]
		} else if((town_count + city_count) % Internal.RoundFloatToInteger((maximum_towns + maximum_cities).tofloat() / maximum_cities.tofloat()) == 0) {
			is_city = true;
			town_size = GSTown.TOWN_SIZE_LARGE;
		
		// Else town, and set town size.
		} else {
			is_city = false;
			switch(minimum_town_size + GSBase.RandRange(3 - minimum_town_size)) {
			
				// Small.
				case 0:
					town_size = GSTown.TOWN_SIZE_SMALL;
				break;
				
				// Medium.
				case 1:
					town_size = GSTown.TOWN_SIZE_MEDIUM;
				break;
				
				// Large.
				case 2:
					town_size = GSTown.TOWN_SIZE_LARGE;
				break;
				
				// Error.
				default:
					throw("MapConstructor.BuildTown: Unknown town size!");
			}
		}
	
		// Switch for build method.
		switch(town_build_method) {
			
			// Metropolis.
			case 1:
				
				// Display msg if first build.
				if((town_count + city_count) == 0) {
					Log.Info("> Building towns/cities using Metropolis method.", Log.SUB);
					Log.Info(">  Each city will be surrounded by " + Internal.RoundFloatToInteger((maximum_towns + maximum_cities).tofloat() / maximum_cities.tofloat()) + " towns.", Log.SUB);
					
				}
				
				// Build
				town_index = TownMetropolisBuildMethod(town_size, is_city, town_layout);
				
			break;
			
			// Clusters.
			case 2:
			
				// Display msg if first build.
				if((town_count + city_count) == 0)
					Log.Info("> Building towns/cities using Clustered method.", Log.SUB);
				
				// Build
				town_index = TownClusteredBuildMethod(town_size, is_city, town_layout);
			break;
			
			// Scattered.
			case 3:	
			
				// Display msg if first build.
				if((town_count + city_count) == 0)
					Log.Info("> Building towns/cities using Scattered method.", Log.SUB);
				
				// Build
				town_index = TownScatteredBuildMethod(town_size, is_city, town_layout);
			break;
			
			// Random.
			case 4:
				
				// Display msg if first build.
				if((town_count + city_count) == 0)
					Log.Info("> Building towns/cities using Random method.", Log.SUB);
				
				// Build
				town_index = TownRandomBuildMethod(town_size, is_city, town_layout);
			break;
			
			default:
				throw("MapConstructor.BuildTown: Unknown build method!");
		}		
		
		// Check for error
		if(town_index == null)
			return;
		
		// If city.
		if(is_city) {
			
			// Multiply size with loop.
			local city_multiplier = GSGameSettings.GetValue("economy.initial_city_size");
			for(local j = 0; j < city_multiplier; j++)
				GSTown.ExpandTown(town_index, GSTown.GetHouseCount(town_index));
			
			// Set count
			city_count++;
			
		// Else town
		} else {
			
			// Set count
			town_count++;
		}
		
	} // End build loop.
	
	// Display status msg
	Log.Info("> Built " + town_count + " towns and " + city_count + " cities, for a total of " + GSTown.GetTownCount() + ".", Log.SUB);
	return;
}

/**
 *	Metropolis build method function for towns. (2).
 *	Builds in customizable clusters, with a city in the center.
 *	@param town_size The GSTown.TownSize sized town to build.
 *	@param is_city True/False to build a city.
 *	@param town_layout The GSTown.RoadLayout road layout to build.
 *	@return Town ID if succeed, null if fail.
 *	@note First must always be city, and doesn't account for distributing these independently.
 */
function MapConstructor::TownMetropolisBuildMethod(town_size, is_city, town_layout) {
	
	// Get last node or add a new one for city.
	local metropolis_node_index = null;
	local metropolis_node_array = null;
	// - Get last node if not a city.	
	if(!is_city) {
		
		local smallest_array = null;
		foreach(current_metropolis_node_index, current_metropolis_node_array in this.town_metropolis_table) {
			
			if(smallest_array == null) {
				smallest_array = current_metropolis_node_array;
				metropolis_node_index = current_metropolis_node_index;
				metropolis_node_array = current_metropolis_node_array;				
			}
				
			if(current_metropolis_node_array.len() < smallest_array.len()) {
				metropolis_node_index = current_metropolis_node_index;
				metropolis_node_array = current_metropolis_node_array;
			}

			//Log.Info(metropolis_node_index + " : " + metropolis_node_array.len(), Log.DETAIL);
		}
	
	// - Else get new node.
	} else {
		
		for(local node_tries = 0 + this.town_metropolis_node_tries; node_tries > 0; node_tries--) {
			
			// Error handler.
			if(node_tries == (this.town_metropolis_node_tries * 0.75).tointeger()) {
				Log.Warning(" Metropolis node tries left: " + node_tries + ".", Log.MAIN);
			} else if(node_tries == (this.town_metropolis_node_tries * 0.5).tointeger()) {
				Log.Error(" Metropolis node tries left: " + node_tries + ".", Log.MAIN);
			} else if(node_tries == (this.town_metropolis_node_tries * 0.25).tointeger()) {
				this.WarningBox("MapConstructor seems to be getting stuck on Metropolis node find for town/city" + 
								"Random tiles tried so far: " + (this.town_metropolis_node_tries - node_tries) + ". " + 
								"See the GS log for details and the Readme for tips how to resolve.");
			}
			
			// Get a random node.
			metropolis_node_index = this.GetRandomTile();
			
			// Node constraints.
			// - Not on water.
			if(GSTile.IsWaterTile(metropolis_node_index))
				continue;
			// - Distance to other nodes.
			local too_close = false;
			foreach(current_metropolis_node_index, current_metropolis_node_array in this.town_metropolis_table) {
				
				if(GSMap.DistanceManhattan(metropolis_node_index, current_metropolis_node_index) < GSController.GetSetting("metropolis_town_distance_nodes")) {
					too_close = true;
					break;
				}
				
			}
			if(too_close)
				continue;
			
			// Create new tile array and break;
			metropolis_node_array = [];
			Log.Info(" ~Created new node " + metropolis_node_index + ", after " + (this.town_metropolis_node_tries - node_tries) + " tries.", Log.DETAIL);
			break;
		}
	}
	
	// - Check if failed to get node.
	if(metropolis_node_array == null) {
		Log.Error("> Failed to find a node!", Log.MAIN);
		return 0;
	}
	
	//GSSign.BuildSign(metropolis_node_index, "TownNode: Metropolis " + metropolis_node_index);
	
	// Loop random clustered tile until correct.
	local tile_index = null;
	for(local build_tries = 0 + this.town_metropolis_build_tries; build_tries > 0; build_tries--) {
		
		// Error handler.
		if(build_tries == (this.town_metropolis_build_tries * 0.75).tointeger()) {
			Log.Warning(" Metropolis build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.town_metropolis_build_tries * 0.5).tointeger()) {
			Log.Error(" Metropolis build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.town_metropolis_build_tries * 0.25).tointeger()) {
			this.WarningBox("MapConstructor seems to be getting stuck on Metropolis build for town/city." + 
							"Random tiles tried so far: " + (this.town_metropolis_build_tries - build_tries) + ". " + 
							"See the GS log for details and the Readme for tips how to resolve.");
			this.ErrorHandler(GSError.GetLastError());
		}
		
		// Set distance from node, much closer if the initial center city.
		local distance_to_node = null;
		if(is_city) {
			distance_to_node = 10;
			
		} else {
			distance_to_node = GSController.GetSetting("metropolis_town_distance_maximum") * (max(1, metropolis_node_array.len() / 6));
		}
		
		// Get a random tile close to node. NOTE USES MODIFIER BASED ON NUMBER OF TOWNS TO USE PER NODE.
		tile_index = this.GetRandomTileCloseToTile(metropolis_node_index, distance_to_node);
		
		// Metropolis constraints.
		// - Distance to cluster town.
		if(GSTown.GetDistanceManhattanToTile(GSTile.GetClosestTown(tile_index),tile_index) < GSController.GetSetting("metropolis_town_distance_minimum"))
			continue;
		
		// Try build.
		if(GSTown.FoundTown(tile_index, town_size, is_city, town_layout, null)) {
			metropolis_node_array.append(tile_index);
			this.town_metropolis_table.rawset(metropolis_node_index, metropolis_node_array);
			if(is_city)
				Log.Info(" ~ Built city: " + GSTown.GetName(GSTile.GetTownAuthority(tile_index)) + ", node: " + metropolis_node_index + ", count: " + metropolis_node_array.len() +  ".", Log.DETAIL);
			else
				Log.Info(" ~ Built town: " + GSTown.GetName(GSTile.GetTownAuthority(tile_index)) + ", node: " + metropolis_node_index + ", count: " + metropolis_node_array.len() +  ".", Log.DETAIL);
			return GSTile.GetTownAuthority(tile_index);		
		}
	}
	
	// Failed to find a tile.
	Log.Error("> Failed to Metropolis build town!", Log.MAIN);
	return 0;
}

/**
 *	Clustered build method function for towns. (2).
 *	Builds in customizable clusters, at distance from towns on the map.
 *	@param town_size The GSTown.TownSize sized town to build.
 *	@param is_city True/False to build a city.
 *	@param town_layout The GSTown.RoadLayout road layout to build.
 *	@return Town ID if succeed, null if fail.
 */
function MapConstructor::TownClusteredBuildMethod(town_size, is_city, town_layout) {

	// Get unfilled node or add a new one.
	// - Check for unfilled node.
	local cluster_node_index = null;
	local cluster_node_array = null
	foreach(current_cluster_node_index, current_cluster_node_array in this.town_cluster_table) {
	
		if(current_cluster_node_array.len() < GSController.GetSetting("clustered_town_nodes")) {
			cluster_node_index = current_cluster_node_index;
			cluster_node_array = current_cluster_node_array;
			break;
		}
	}
	
	// - Get new node if required.
	if(cluster_node_index == null) {
		
		for(local node_tries = 0 + this.town_clustered_node_tries; node_tries > 0; node_tries--) {
			
			// Error handler.
			if(node_tries == (this.town_clustered_node_tries * 0.75).tointeger()) {
				Log.Warning(" Clustered node tries left: " + node_tries + ".", Log.MAIN);
			} else if(node_tries == (this.town_clustered_node_tries * 0.5).tointeger()) {
				Log.Error(" Clustered node tries left: " + node_tries + ".", Log.MAIN);
			} else if(node_tries == (this.town_clustered_node_tries * 0.25).tointeger()) {
				this.WarningBox("MapConstructor seems to be getting stuck on Clustered node find for town/city" + 
								"Random tiles tried so far: " + (this.town_clustered_node_tries - node_tries) + ". " + 
								"See the GS log for details and the Readme for tips how to resolve.");
			}
			
			// Get a random node.
			cluster_node_index = this.GetRandomTile();
			
			// Node constraints.
			// - Not on water.
			if(GSTile.IsWaterTile(cluster_node_index))
				continue;
			// - Distance to other nodes.
			local too_close = false;
			foreach(current_cluster_node_index, current_cluster_node_array in this.town_cluster_table) {
				
				if(GSMap.DistanceManhattan(cluster_node_index, current_cluster_node_index) < GSController.GetSetting("clustered_town_distance_nodes")) {
					too_close = true;
					break;
				}
				
			}
			if(too_close)
				continue;
			
			// Create new tile array and break;
			cluster_node_array = [];
			Log.Info(" ~Created new node " + cluster_node_index + ", after " + (this.town_clustered_node_tries - node_tries) + " tries.", Log.DETAIL);
			break;
		}
	}
	
	// - Check if failed to get node.
	if(cluster_node_array == null) {
		Log.Error("> Failed to find a node!", Log.MAIN);
		return 0;
	}
	
	//GSSign.BuildSign(cluster_node_index, "TownNode: Cluster " + cluster_node_index);
	
	// Loop random clustered tile until correct.
	local tile_index = null;
	for(local build_tries = 0 + this.town_clustered_build_tries; build_tries > 0; build_tries--) {
		
		// Error handler.
		if(build_tries == (this.town_clustered_build_tries * 0.75).tointeger()) {
			Log.Warning(" Clustered build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.town_clustered_build_tries * 0.5).tointeger()) {
			Log.Error(" Clustered build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.town_clustered_build_tries * 0.25).tointeger()) {
			this.WarningBox("MapConstructor seems to be getting stuck on Clustered build for town/city." + 
							"Random tiles tried so far: " + (this.town_clustered_build_tries - build_tries) + ". " + 
							"See the GS log for details and the Readme for tips how to resolve.");
			this.ErrorHandler(GSError.GetLastError());
		}
		
		// Get a random tile close to node. NOTE USES MODIFIER BASED ON NUMBER OF TOWNS TO USE PER NODE.
		tile_index = this.GetRandomTileCloseToTile(cluster_node_index, GSController.GetSetting("clustered_town_distance_maximum") * (max(1, cluster_node_array.len() / 6)));
		
		// Clustered constraints.
		// - Distance to cluster town.
		if(GSTown.GetDistanceManhattanToTile(GSTile.GetClosestTown(tile_index),tile_index) < GSController.GetSetting("clustered_town_distance_minimum"))
			continue;
		
		// Try build.
		if(GSTown.FoundTown(tile_index, town_size, is_city, town_layout, null)) {
			cluster_node_array.append(tile_index);
			this.town_cluster_table.rawset(cluster_node_index, cluster_node_array);
			Log.Info(" ~ Built: " + GSTown.GetName(GSTile.GetTownAuthority(tile_index)) + ", node: " + cluster_node_index + ", count: " + cluster_node_array.len() +  ".", Log.DETAIL);
			return GSTile.GetTownAuthority(tile_index);		
		}
	}
	
	// Failed to find a tile.
	Log.Error("> Failed to Clustered build town!", Log.MAIN);
	return 0;
}

/**
 *	Scattered build method function for towns. (3).
 *	Builds scattered, at distance from towns on the map.
 *	@param town_size The GSTown.TownSize sized town to build.
 *	@param is_city True/False to build a city.
 *	@param town_layout The GSTown.RoadLayout road layout to build.
 *	@return Town ID if succeed, null if fail.
 */
function MapConstructor::TownScatteredBuildMethod(town_size, is_city, town_layout) {

	// Loop random scattered tile until correct.
	local tile_index = null;
	for(local build_tries = 0 + this.town_scattered_build_tries; build_tries > 0; build_tries--) {
	
		// Error handler.
		if(build_tries == (this.town_scattered_build_tries * 0.75).tointeger()) {
			Log.Warning(" Random build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.town_scattered_build_tries * 0.5).tointeger()) {
			Log.Error(" Random build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.town_scattered_build_tries * 0.25).tointeger()) {
			this.WarningBox("MapConstructor seems to be getting stuck on Scattered build for town/city. " + 
							"Random tiles tried so far: " + (this.town_scattered_build_tries - build_tries) + ". " + 
							"See the GS log for details and the Readme for tips how to resolve.");
			this.ErrorHandler(GSError.GetLastError());
		}
		
		// Get a random tile.
		tile_index = this.GetRandomTile();
		
		// Scattered constraints.
		// - Distance to town.
		if(GSTown.GetDistanceManhattanToTile(GSTile.GetClosestTown(tile_index),tile_index) < GSController.GetSetting("scattered_town_town"))
			continue;
		
		// Try build.
		if(GSTown.FoundTown(tile_index, town_size, is_city, town_layout, null))
			return GSTile.GetTownAuthority(tile_index);
			
	}
	
	// Failed to find a tile.
	Log.Error("> Failed to Scattered build town/city!", Log.MAIN);
	return null;
}

/**
 *	Random build method function for towns. (4).
 *	Builds randomly on the map.
 *	@param town_size The GSTown.TownSize sized town to build.
 *	@param is_city True/False to build a city.
 *	@param town_layout The GSTown.RoadLayout road layout to build.
 *	@return Town ID if succeed, null if fail.
 */
function MapConstructor::TownRandomBuildMethod(town_size, is_city, town_layout) {
	
	// Loop random tile until correct.
	local tile_index = null;
	for(local build_tries = 0 + this.town_random_build_tries; build_tries > 0; build_tries--) {
	
		// Error handler.
		if(build_tries == (this.town_random_build_tries * 0.75).tointeger()) {
			Log.Warning(" Random build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.town_random_build_tries * 0.5).tointeger()) {
			Log.Error(" Random build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.town_random_build_tries * 0.25).tointeger()) {
			this.WarningBox("MapConstructor seems to be getting stuck on Random build for town/city. " + 
							"Random tiles tried so far: " + (this.town_random_build_tries - build_tries) + ". " + 
							"See the GS log for details and the Readme for tips how to resolve.");
			this.ErrorHandler(GSError.GetLastError());
		}
		
		// Get a random tile.
		tile_index = this.GetRandomTile();
		
		// Try build.
		if(GSTown.FoundTown(tile_index, town_size, is_city, town_layout, null))
			return GSTile.GetTownAuthority(tile_index);
			
	}
	
	// Failed to find a tile.
	Log.Error("> Failed to Random build town/city!", Log.MAIN);
	return null;
}

/**
 *	The function to build industries initially and each refresh period.
 *	Runs through each industry class, checks if it has built that class already, builds random class once if not and sets to true, 
 *		if all have been built resets all to false up until global max per refresh
 *	Builds a global set proportion of each industry class, minimum 1 of each industry.
 *	Checks how many of each industry are existing (if 0 then has a chance to build 1).
 *	Builds up to a random number of each industry, between min and max according to proportion.
 *	Builds each industry according to build method.
 *	Accounts for special industry types.
 */
function MapConstructor::BuildIndustry() {
	
	// Display msg.
	Log.Info("+==============================+", Log.MAIN);
	Log.Info("Building industries...", Log.MAIN);
	
	// Build limits.
	local current_build_limit = 0;			// Build limit in this function
	local current_ind_max = 0.0;			// Max factor in this function		
	
	// If first build has not happened, then this is a new game.
	if(!this.first_industry_build_performed) {
	
		// If global build count is less than 1 minimum per industry set to 1 each, else set to global maximum.
		if(this.density_industries_total < this.industry_type_list.Count())
			current_build_limit = this.industry_type_list.Count();
		else
			current_build_limit = this.density_industries_total;
		
		// Set maximum factor to 1.
		current_ind_max = 1.0;
		
		// Display status msg.
		Log.Info("> This is a new game, industry build limit is: " + current_build_limit + ".", Log.SUB);
		
		// Set first build
		this.first_industry_build_performed = true;
		
	// Else build has happened, and we will build a few only.
	} else {
	
		// Set build limit to a random up to global.
		current_build_limit = GSBase.RandRange(this.build_industries_limit + 1);
		
		// Set max factor to global.
		current_ind_max = this.density_industries_maximum;
		
		// Display status msg.
		Log.Info("> This is a continuing game, industry build limit is: " + current_build_limit + " / " + this.build_industries_limit + " (Max).", Log.SUB);
	}
	
	// Check to continue.
	if(current_build_limit == 0)
		return;
	
	// Build loop variables.
	local current_build_count = 0;					// Total build count for each this build function.
	local current_method = null;					// Current build method for each loop.
	local current_industry_proportion = null;		// Current industry proportion for each loop.
	local current_industry_array = null;			// Current industry list for each loop.
	
	// Loop while below current build limit.
	local current_build_target = 0;
	while(current_build_count < current_build_limit) {
		
		// Skip empty classes.
		if(this.raw_industry_array.len() == 0)
			this.primary_performed = true;
		if(this.processor_industry_array.len() == 0)
			this.secondary_performed = true;
		if(this.tertiary_industry_array.len() == 0)
			this.tertiary_performed = true;
		if(this.special_industry_array.len() == 0)
			this.special_performed = true;		
		
		// Build "town" methods first, else select a random industry class not yet built.
		// 0 = raw, 1 = processor, 2 = tertiary, 3 = special.
		local current_industry_class = null;
		if(!this.primary_performed && this.density_industries_raw_method == 1)
			current_industry_class = 0;
		else if(!this.secondary_performed && this.density_industries_processor_method == 1)
			current_industry_class = 1;
		else if(!this.tertiary_performed && this.density_industries_tertiary_method == 1)
			current_industry_class = 2;
		else
			current_industry_class = GSBase.RandRange(4);
			
		// Switch for industry class.
		local current_class_name = null;
		switch(current_industry_class) {
			
			// Raw.
			case 0:
				
				// Skip if performed already.
				if(this.primary_performed)
					continue;
				
				// Assign current class.
				current_class_name = "primary";
				current_method = this.density_industries_raw_method;
				current_industry_proportion = this.density_industries_raw;
				current_industry_array = this.raw_industry_array;
				
				this.primary_performed = true;
			break;
			
			// Processor.
			case 1:
			
				// Skip if performed already.
				if(this.secondary_performed)
					continue;
				
				// Assign current class.
				current_class_name = "secondary";
				current_method = this.density_industries_processor_method;
				current_industry_proportion = this.density_industries_processor;
				current_industry_array = this.processor_industry_array;
				
				this.secondary_performed = true;
			break;
			
			// Tertiary.
			case 2:
			
				// Skip if performed already.
				if(this.tertiary_performed)
					continue;
				
				// Skip empty class.
				if(this.tertiary_industry_array.len() == 0)
					continue;
				
				// Assign current class.
				current_class_name = "tertiary";
				current_method = this.density_industries_tertiary_method;
				current_industry_proportion = this.density_industries_tertiary;
				current_industry_array = this.tertiary_industry_array;
				
				this.tertiary_performed = true;
			break;
			
			// Special.
			case 3:
			
				// Skip if performed already.
				if(this.special_performed)
					continue;
				
				// Skip empty class.
				if(this.special_industry_array.len() == 0)
					continue;
				
				// Assign current class.
				current_class_name = "special";
				current_method = this.density_industries_special_method;		// Special case non-selectable
				current_industry_proportion = this.density_industries_special;
				current_industry_array = this.special_industry_array;
				
				this.special_performed = true;
			break;
			
			// Error.
			default:
				throw("MapConstructor::BuildIndustry: Unknown current_industry_class!");
		}
		
		// Display msg.
		Log.Info("+------------------------------+", Log.MAIN);
		Log.Info("> Building " + current_class_name + " industries.", Log.MAIN);
		
		// Set densities
		// Mins are how much a map will start with and the start count for building extra.
		// Maxes are how much a map could get based on chance. At least 1 per industry.

		local class_minimum	= 0;					// Integer, could be 0.
		local class_maximum = 0;					// Integer, at least 1 per industry and class_minimum.
		local industry_minimum = 0;					// Integer truncated float, could be 0.
		local industry_maximum = 0;					// Integer truncated float, always >= 1.
		
		// Min density per industry class is:
		class_minimum = (this.density_industries_total * current_industry_proportion * this.density_industries_minimum).tointeger();
		Log.Info(" ~Industry class minimum is: " + class_minimum + ".", Log.SUB);
		// Max density per industry class is:
		class_maximum = max(current_industry_array.len(), (this.density_industries_total * current_industry_proportion * current_ind_max).tointeger()); 
		Log.Info(" ~Industry class maximum is: " + class_maximum + ".", Log.SUB);
		
		// Min per industry is:
		industry_minimum = class_minimum / current_industry_array.len();	
		Log.Info(" - Industry type minimum is: " + industry_minimum + ".", Log.DETAIL);
		// Max per industry is:
		industry_maximum = class_maximum / current_industry_array.len();	
		Log.Info(" - Industry type maximum is: " + industry_maximum + ".", Log.DETAIL);
		
		// Loop through each industry type in current class array.
		local current_class_build_target = 0;
		local current_class_build_count = 0;
		foreach(current_industry_type in current_industry_array) {
			
			// Display status
			Log.Info("", Log.DETAIL);
			Log.Info(" - Analyzing " + GSIndustryType.GetName(current_industry_type) + " (ID:  " + current_industry_type + ").", Log.DETAIL);
				
			// Count existing for industry type.
			local existing_industry_list = GSIndustryList();
			existing_industry_list.Valuate(GSIndustry.GetIndustryType);
			existing_industry_list.KeepValue(current_industry_type);
			local existing_industry_count = existing_industry_list.Count();
			
			Log.Info(" - Existing " + GSIndustryType.GetName(current_industry_type) + " count is: " + existing_industry_count + ".", Log.DETAIL);
			
			// Set build target, a random target for each function to build. 
			local current_industry_build_target = min((current_build_limit - current_build_count),
											  (industry_minimum + (GSBase.RandRange(industry_maximum - industry_minimum + 1)) - existing_industry_count));
			
			// Skip if max is built already.
			if(current_industry_build_target <= 0)
				continue;
			
			// Update targets.
			current_class_build_target += current_industry_build_target;
			current_build_target += current_industry_build_target;
			
			// Display status
			Log.Info(" - Building " + current_industry_build_target + " " + GSIndustryType.GetName(current_industry_type) + "(s).", Log.DETAIL);
			
			//Loop for each industry to build.
			local current_industry_build_count = 0;			// How many were actually built for this industry in this build run.
			for(local i = 0; i < current_industry_build_target; i++) {
			
				// Build switch for method.
				switch(current_method) {
				
					// Increment count using town build.
					case 1:
						// Display info msg
						if(i == 0)
							Log.Info(" - Using Town method.", Log.DETAIL);
						
						current_industry_build_count += this.IndustryTownBuildMethod(current_industry_type);
					break;
					
					// Increment count using clustered build.
					case 2:
						// Display info msg
						if(i == 0)
							Log.Info(" - Using Clustered method.", Log.DETAIL);
							
						current_industry_build_count += this.IndustryClusteredBuildMethod(current_industry_type);
					break;
					
					// Increment count using scatter build
					case 3:
						// Display info msg
						if(i == 0)
							Log.Info(" - Using Scattered method.", Log.DETAIL);
						
						current_industry_build_count += this.IndustryScatteredBuildMethod(current_industry_type);
					break;
					
					// Increment count using random build.
					case 4:
						// Display info msg
						if(i == 0)
							Log.Info(" - Using Random method.", Log.DETAIL);
							
						current_industry_build_count += this.IndustryRandomBuildMethod(current_industry_type);
					break;
					
					// Increment count using special build.
					case 5:
						// Display info msg
						if(i == 0)
							Log.Info(" - Using special method.", Log.DETAIL);
						
						current_industry_build_count += this.IndustrySpecialBuildMethod(current_industry_type);
					break;
					
					default:
						throw("MapConstructor.BuildIndustry: Unknown build method!");
				}
				
			} // End industry for loop.
			
			// Display status
			Log.Info(" - Built " + current_industry_build_count + " / " + current_industry_build_target + ".", Log.DETAIL);
			
			// Update build count.
			current_class_build_count += current_industry_build_count;
			current_build_count += current_industry_build_count;
			
		} // End industry type while loop.
		
		// Display status.
		Log.Info(" ~Built " + current_class_build_count + " / " + current_class_build_target + " " + current_class_name + " industries.", Log.SUB);
		
		// Reset random if all have been done.
		if(this.primary_performed && this.secondary_performed && this.tertiary_performed && this.special_performed) {
			this.primary_performed = false;
			this.secondary_performed = false;
			this.tertiary_performed = false;
			this.special_performed = false;
			
			// Display status.
			Log.Info("+------------------------------+", Log.MAIN);
			Log.Info("> Built a total of: " + current_build_count + " / " + current_build_target + ".", Log.SUB);
			Log.Info("> Built all industry class types.", Log.MAIN);
			
			return;
		}
		
	} // End class while loop.
	
}

/**
 *	Town build method function for industries. (1).
 *	Builds close to towns, at distance from towns and/or industries, on the map.
 *	@param industry_type_id The industry ID to build.
 *	@return 1 if built and 0 if not.
 */
 // TODO: Error and build count handle.
function MapConstructor::IndustryTownBuildMethod(industry_type_id) {
	
	//Check if industry is not build-able.
	if(!GSIndustryType.CanBuildIndustry(industry_type_id))
		throw("MapConstructor::IndustryTownBuildMethod: Industry not build-able!");
	
	// Check abnormal industries.
	local abnormal = this.IndustryAbnormalCheck(GSIndustryType.GetName(industry_type_id));
	
	// Prospect if abnormal and required.
	if((abnormal != null) && (GSController.GetSetting("abnormal_industries_prospect") == 1)) {
		
		// Try prospect
		if(GSIndustryType.ProspectIndustry(industry_type_id)) {
			Log.Info(" -> Prospected.", Log.DETAIL);
			return 1;
		}
		
		// Error handler.
		this.ErrorHandler(GSError.GetLastError());
		
		return 0;
	}
	
	// Create town list for above population setting.
	local town_node_list = GSTownList();
	town_node_list.Valuate(GSTown.GetPopulation);
	town_node_list.RemoveBelowValue(GSController.GetSetting("town_industry_minimum_population"));
	
	// Abnormal constraints.
	if(abnormal != null) {
		
		// Distance from edge, TIMES 2.
		if(abnormal[0] != null) {
			town_node_list.Valuate(function(town_index) { return GSMap.DistanceFromEdge(GSTown.GetLocation(town_index)) } );
			town_node_list.KeepBelowValue(abnormal[0] * 2);
		}
		
		// Terrain type preference.
		//if((abnormal[1] != null) {
		//	town_node_list.Valuate(function(town_index) { return GSTile.GetTerrainType(GSTown.GetLocation(town_index)) } );
		//	town_node_list.KeepValue(abnormal[1]);
		//}
	}
	
	// Check if empty.
	if(town_node_list.IsEmpty()) {
		Log.Warning(" ~No valid towns above population requirement!", Log.SUB);
		return 0;
	}
	
	// Town constraints.
	// - Maximum industries per town.
	town_node_list.Valuate(function(town_index) { return 0 } );
	local industry_list = List(GSIndustryList());
	industry_list.Valuate(function(industry_index) { return GSTile.GetTownAuthority(GSIndustry.GetLocation(industry_index)) } );
	foreach(industry_index, town_index in industry_list) {
		
		local town_industry_count = town_node_list.GetValue(town_index);
		town_node_list.SetValue(town_index, town_industry_count++);
	}
	town_node_list.RemoveAboveValue(GSController.GetSetting("town_industry_maximum_industries"));
	
	// - Multiple industries per town.
	if(GSGameSettings.GetValue("multiple_industry_per_town") == 0) {
		
		// List of industries of same type valuated by town.
		industry_list.Valuate(GSIndustry.GetIndustryType);
		industry_list.KeepValue(industry_type_id);
		industry_list.SetItemAsValue();
		
		town_node_list.RemoveList(industry_list);
	}
	
	// Check if empty.
	if(town_node_list.IsEmpty()) {
		Log.Warning(" ~No valid towns with industry limit requirements!", Log.SUB);
		return 0;
	}
	
	// Loop through town list.
	town_node_list.Valuate(GSBase.RandItem);
	town_node_list.Sort(GSList.SORT_BY_VALUE, GSList.SORT_DESCENDING);
	for(local current_town_node = town_node_list.Begin(); !town_node_list.IsEnd(); current_town_node = town_node_list.Next()) {
		
		// Display msg.
		Log.Info(" - Trying town " + GSTown.GetName(current_town_node) + ".", Log.DETAIL);
		
		// Assign radius. USES HEURISTIC!
		local town_radius = (GSTown.GetHouseCount(current_town_node).tofloat() * 0.90).tointeger();
		
		// Get random tile list of square around town.
		local town_tile_list = SuperLib.Tile.MakeTileRectAroundTile(GSTown.GetLocation(current_town_node), town_radius);
		town_tile_list.Valuate(GSTile.GetTownAuthority);
		town_tile_list.KeepValue(current_town_node);
		town_tile_list.Valuate(GSBase.RandItem);
		town_tile_list.Sort(GSList.SORT_BY_VALUE, GSList.SORT_DESCENDING);
		
		// Loop for each tile.
		local build_tries = 0;
		foreach(tile_index, _ in town_tile_list) {
		
			build_tries++;
		
			// Abnormal constraints.
			if(abnormal != null) {
				
				// Distance from edge.
				if((abnormal[0] != null) && (GSMap.DistanceFromEdge(tile_index) > abnormal[0]))
					continue;
				
				// Terrain type preference.
				if((abnormal[1] != null) && (GSTile.GetTerrainType(tile_index) != abnormal[1]))
					continue;
			}
			
			// Town constraints.
			// - Distance to industry.
			local closest_industry = this.GetClosestIndustry(tile_index);
			if((closest_industry != null) && (GSTile.GetDistanceManhattanToTile(GSIndustry.GetLocation(closest_industry), tile_index) < GSController.GetSetting("town_industry_distance_industry")))
				continue;
			
			// Try build.
			if(GSIndustryType.BuildIndustry(industry_type_id, tile_index)) {
				Log.Info(" -> Built.", Log.DETAIL);
				return 1;
			}
		}
		
		// Errorhandler.
		//this.ErrorHandler(GSError.GetLastError());
		
	} // End town list loop.
	
	// Warning msgbox.
	this.WarningBox("MapConstructor seems to be getting stuck on Town build for " + GSIndustryType.GetName(industry_type_id) + ". " + 
					"Towns tried: " + (town_node_list.Count()) + ". " + "See the GS log for details and the Readme for tips how to resolve.");
	
	// Failed to find a tile.
	Log.Error("> Failed to Town build " + GSIndustryType.GetName(industry_type_id) + "!", Log.MAIN);
	return 0;
}

/**
 *	Clustered build method function for industries. (2).
 *	Builds in customizable clusters, at distance from towns and/or industries, on the map.
 *	@param industry_type_id The industry ID to build.
 *	@return 1 if built and 0 if not.
 */
 // Check if multi_industries per town is allowed.
function MapConstructor::IndustryClusteredBuildMethod(industry_type_id) {
	
	//Check if industry is not build-able.
	if(!GSIndustryType.CanBuildIndustry(industry_type_id))
		throw("MapConstructor::IndustryClusteredBuildMethod: Industry not build-able!");
	
	// Check abnormal industries.
	local abnormal = this.IndustryAbnormalCheck(GSIndustryType.GetName(industry_type_id));
	
	// Prospect if abnormal and required.
	if((abnormal != null) && (GSController.GetSetting("abnormal_industries_prospect") == 1)) {
		
		// Try prospect
		if(GSIndustryType.ProspectIndustry(industry_type_id)) {
			Log.Info(" -> Prospected.", Log.DETAIL);
			return 1;
		}
		
		// Error handler.
		this.ErrorHandler(GSError.GetLastError());
		
		return 0;
	}
	
	// Get industry type node data from table.			//{industry_type_id, {node_id, [tile_index, tile_index ...(max_per_cluster)]}}
	if(!this.industry_cluster_table.rawin(industry_type_id)) {
		this.industry_cluster_table.rawset(industry_type_id, {})
	}
	local industry_type_cluster_table = this.industry_cluster_table.rawget(industry_type_id);
	
	// Get unfilled node or add a new one.
	// - Check for unfilled node.
	local cluster_node_index = null;
	local cluster_node_array = null
	foreach(current_cluster_node_index, current_cluster_node_array in industry_type_cluster_table) {
	
		if(current_cluster_node_array.len() < GSController.GetSetting("clustered_industry_nodes")) {
			cluster_node_index = current_cluster_node_index;
			cluster_node_array = current_cluster_node_array;
			break;
		}
	}
	
	// - Get new node if required.
	if(cluster_node_index == null) {
		
		for(local node_tries = 0 + this.industry_clustered_node_tries; node_tries > 0; node_tries--) {
			
			// Error handler.
			if(node_tries == (this.industry_clustered_node_tries * 0.75).tointeger()) {
				Log.Warning(" Clustered node tries left: " + node_tries + ".", Log.MAIN);
			} else if(node_tries == (this.industry_clustered_node_tries * 0.5).tointeger()) {
				Log.Error(" Clustered node tries left: " + node_tries + ".", Log.MAIN);
			} else if(node_tries == (this.industry_clustered_node_tries * 0.25).tointeger()) {
				this.WarningBox("MapConstructor seems to be getting stuck on Clustered node find for " + GSIndustryType.GetName(industry_type_id) + ". " + 
								"Random tiles tried so far: " + (this.industry_clustered_node_tries - node_tries) + ". " + 
								"See the GS log for details and the Readme for tips how to resolve.");
			}
			
			// Get a random node.
			cluster_node_index = this.GetRandomTile();
			
			// Abnormal constraints.
			if(abnormal != null) {
				
				// Distance from edge.
				if(abnormal[0] != null)
					cluster_node_index = this.GetRandomTileCloseToEdge(abnormal[0]);
				
				// Terrain type preference.
				if((abnormal[1] != null) && (GSTile.GetTerrainType(cluster_node_index) != abnormal[1]))
					continue;
			}
			
			// Node constraints.
			// - Not on water, even if industry is a water industry!
			if(GSTile.IsWaterTile(cluster_node_index))
				continue;
			// - Distance to town.
			if(GSTown.GetDistanceManhattanToTile(GSTile.GetClosestTown(cluster_node_index),cluster_node_index) < GSController.GetSetting("clustered_industry_distance_towns"))
				continue;
			// - Distance to industry.
			local closest_industry = this.GetClosestIndustry(cluster_node_index);
			if((closest_industry != null) && (GSTile.GetDistanceManhattanToTile(GSIndustry.GetLocation(closest_industry), cluster_node_index) < GSController.GetSetting("clustered_industry_distance_industries")))
				continue;
			// - Distance to other nodes.
			local too_close = false;
			foreach(current_industry_type_id, current_industry_type_cluster_table in this.industry_cluster_table) {
			
				foreach(current_cluster_node_index, current_cluster_node_array in current_industry_type_cluster_table) {
					
					if(GSTile.GetDistanceManhattanToTile(cluster_node_index, current_cluster_node_index) < GSController.GetSetting("clustered_industry_distance_nodes")) {
						too_close = true;
						break;
					}
					
				}
				
				if(too_close)
					break;
			}
			
			if(too_close)
				continue;
			
			// Create new tile array and break;
			cluster_node_array = [];
			break;
		}
	}
	
	// - Check if failed to get node.
	if(cluster_node_array == null) {
		Log.Error("> Failed to find a node!", Log.MAIN);
		return 0;
	}
	
	// GSSign.BuildSign(cluster_node_index, "Node: " + GSIndustryType.GetName(industry_type_id));
	
	// Loop random clustered tile until correct.
	local tile_index = null;
	for(local build_tries = 0 + this.industry_clustered_build_tries; build_tries > 0; build_tries--) {
		
		// Error handler.
		if(build_tries == (this.industry_clustered_build_tries * 0.75).tointeger()) {
			Log.Warning(" Clustered build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.industry_clustered_build_tries * 0.5).tointeger()) {
			Log.Error(" Clustered build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.industry_clustered_build_tries * 0.25).tointeger()) {
			this.WarningBox("MapConstructor seems to be getting stuck on Clustered build for " + GSIndustryType.GetName(industry_type_id) + ". " + 
							"Random tiles tried so far: " + (this.industry_clustered_build_tries - build_tries) + ". " + 
							"See the GS log for details and the Readme for tips how to resolve.");
			this.ErrorHandler(GSError.GetLastError());
		}
		
		// Get a random tile close to node.
		tile_index = this.GetRandomTileCloseToTile(cluster_node_index, GSController.GetSetting("clustered_industry_distance_maximum") * (max(1, cluster_node_array.len() / 2)));
		
		// Abnormal constraints.
		if(abnormal != null) {
			
			// Distance from edge.
			//if(abnormal[0] != null)
			//	tile_index = this.GetRandomTileCloseToEdge(abnormal[0]);
			
			// Terrain type preference.
			if((abnormal[1] != null) && (GSTile.GetTerrainType(tile_index) != abnormal[1]))
				continue;
		}
		
		// Clustered constraints.
		// - Distance to cluster industry.
		local closest_industry = this.GetClosestIndustry(tile_index);
		if((closest_industry != null) && (GSTile.GetDistanceManhattanToTile(GSIndustry.GetLocation(closest_industry), tile_index) < GSController.GetSetting("clustered_industry_distance_minimum")))
			continue;
		
		// Try build.
		if(GSIndustryType.BuildIndustry(industry_type_id, tile_index)) {
			cluster_node_array.append(tile_index);
			industry_type_cluster_table.rawset(cluster_node_index, cluster_node_array);
			this.industry_cluster_table.rawset(industry_type_id, industry_type_cluster_table);
			Log.Info(" -> Built.", Log.DETAIL);
			return 1;
		}
	}
	
	// Failed to find a tile.
	Log.Error("> Failed to Clustered build " + GSIndustryType.GetName(industry_type_id) + "!", Log.MAIN);
	return 0;
}

/**
 *	Scattered build method function for industries. (3).
 *	Builds scattered, at distance from towns and/or industries, on the map.
 *	@param industry_type_id The industry ID to build.
 *	@return 1 if built and 0 if not.
 */
function MapConstructor::IndustryScatteredBuildMethod(industry_type_id) {
	
	//Check if industry is not build-able.
	if(!GSIndustryType.CanBuildIndustry(industry_type_id))
		throw("MapConstructor::IndustryScatteredBuildMethod: Industry not build-able!");
	
	// Check abnormal industries.
	local abnormal = this.IndustryAbnormalCheck(GSIndustryType.GetName(industry_type_id));
	
	// Prospect if abnormal and required.
	if((abnormal != null) && (GSController.GetSetting("abnormal_industries_prospect") == 1)) {
		
		// Try prospect
		if(GSIndustryType.ProspectIndustry(industry_type_id)) {
			Log.Info(" -> Prospected.", Log.DETAIL);
			return 1;
		}
		
		// Error handler.
		this.ErrorHandler(GSError.GetLastError());
		
		return 0;
	}
	
	// Loop random scattered tile until correct.
	local tile_index = null;
	for(local build_tries = 0 + this.industry_scattered_build_tries; build_tries > 0; build_tries--) {
		
		// Error handler.
		if(build_tries == (this.industry_scattered_build_tries * 0.75).tointeger()) {
			Log.Warning(" Scattered build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.industry_scattered_build_tries * 0.5).tointeger()) {
			Log.Error(" Scattered build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.industry_scattered_build_tries * 0.25).tointeger()) {
			this.WarningBox("MapConstructor seems to be getting stuck on Scattered build for " + GSIndustryType.GetName(industry_type_id) + ". " + 
							"Random tiles tried so far: " + (this.industry_scattered_build_tries - build_tries) + ". " + 
							"See the GS log for details and the Readme for tips how to resolve.");
			this.ErrorHandler(GSError.GetLastError());
		}
		
		// Get a random tile.
		tile_index = this.GetRandomTile();
		
		// Abnormal constraints.
		if(abnormal != null) {
			
			// Distance from edge.
			if(abnormal[0] != null)
				tile_index = this.GetRandomTileCloseToEdge(abnormal[0]);
			
			// Terrain type preference.
			if((abnormal[1] != null) && (GSTile.GetTerrainType(tile_index) != abnormal[1]))
				continue;
		}
		
		// Scattered constraints.
		// - Distance to town.
		if(GSTown.GetDistanceManhattanToTile(GSTile.GetClosestTown(tile_index),tile_index) < GSController.GetSetting("scattered_industry_town"))
			continue;
		// - Distance to industry.
		local closest_industry = this.GetClosestIndustry(tile_index);
		if((closest_industry != null) && (GSTile.GetDistanceManhattanToTile(GSIndustry.GetLocation(closest_industry),tile_index) < GSController.GetSetting("scattered_industry_industry")))
			continue;
		
		// Try build.
		if(GSIndustryType.BuildIndustry(industry_type_id, tile_index)) {
			Log.Info(" -> Built.", Log.DETAIL);
			return 1;
		}
	}
	
	// Failed to find a tile.
	Log.Error("> Failed to Scattered build " + GSIndustryType.GetName(industry_type_id) + "!", Log.MAIN);
	return 0;
}

/**
 *	Random build method function for industries. (4).
 *	Builds randomly on the map.
 *	@param industry_type_id The industry ID to build.
 *	@return 1 if built and 0 if not.
 */
function MapConstructor::IndustryRandomBuildMethod(industry_type_id) {
	
	//Check if industry is not build-able.
	if(!GSIndustryType.CanBuildIndustry(industry_type_id))
		Log.Error("> Failed to build " + GSIndustryType.GetName(industry_type_id) + ": not build-able!", Log.MAIN);
		return 0;
		//throw("MapConstructor::IndustryRandomBuildMethod: Industry not build-able!");
	
	// Check abnormal industries.
	local abnormal = this.IndustryAbnormalCheck(GSIndustryType.GetName(industry_type_id));
	
	// Prospect if abnormal and required.
	if((abnormal != null) && (GSController.GetSetting("abnormal_industries_prospect") == 1)) {
		
		// Try prospect
		if(GSIndustryType.ProspectIndustry(industry_type_id)) {
			Log.Info(" -> Prospected.", Log.DETAIL);
			return 1;
		}
		
		// Error handler.
		this.ErrorHandler(GSError.GetLastError());
		
		return 0;
	}
	
	// Loop random tile until correct.
	local tile_index = null;
	for(local build_tries = 0 + this.industry_random_build_tries; build_tries > 0; build_tries--) {
	
		// Error handler.
		if(build_tries == (this.industry_random_build_tries * 0.75).tointeger()) {
			Log.Warning(" Random build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.industry_random_build_tries * 0.5).tointeger()) {
			Log.Error(" Random build tries left: " + build_tries + ".", Log.MAIN);
			this.ErrorHandler(GSError.GetLastError());
		} else if(build_tries == (this.industry_random_build_tries * 0.25).tointeger()) {
			this.WarningBox("MapConstructor seems to be getting stuck on Random build for " + GSIndustryType.GetName(industry_type_id) + ". " + 
							"Random tiles tried so far: " + (this.industry_random_build_tries - build_tries) + ". " + 
							"See the GS log for details and the Readme for tips how to resolve.");
			this.ErrorHandler(GSError.GetLastError());
		}
		
		// Get a random tile.
		tile_index = this.GetRandomTile();
		
		// Abnormal constraints.
		if(abnormal != null) {
			
			// Distance from edge.
			if(abnormal[0] != null)
				tile_index = this.GetRandomTileCloseToEdge(abnormal[0]);
			
			// Terrain type preference.
			if((abnormal[1] != null) && (GSTile.GetTerrainType(tile_index) != abnormal[1]))
				continue;
		}
		
		// Try build.
		if(GSIndustryType.BuildIndustry(industry_type_id, tile_index)) {
			Log.Info(" -> Built.", Log.DETAIL);
			return 1;
		}
		
	}
	
	// Failed to find a tile.
	Log.Error("> Failed to Random build " + GSIndustryType.GetName(industry_type_id) + "!", Log.MAIN);
	return 0;
}

/**
 *	Special build method function for special industries. (5).
 *	Builds with special conditions or optional "hard code" methods specific for each type.
 *	@param industry_type_id The industry ID to build.
 *	@return 1 if built and 0 if not.
 */
function MapConstructor::IndustrySpecialBuildMethod(industry_type_id) {
	
	//Check if industry is not build-able.
	if(!GSIndustryType.CanBuildIndustry(industry_type_id))
		throw("MapConstructor::IndustrySpecialBuildMethod: Industry not build-able!");
	
	// Switch industry id for each special type, must be same as in this.special_industry_names.
	switch(GSIndustryType.GetName(industry_type_id)) {
		
		// "Bank".
		case this.special_industry_names[0]:
		
			// Check towns with population > setting.
			// - Create town list.
			local bank_town_list = GSTownList();
			// - Remove below population setting.
			bank_town_list.Valuate(GSTown.GetPopulation);
			bank_town_list.RemoveBelowValue(GSController.GetSetting("special_bank"));
			
			// Check if list is empty.
			if(bank_town_list.IsEmpty()) {
				Log.Warning(" - No towns with more than " + GSController.GetSetting("special_bank") + " population, for " + this.special_industry_names[0] + "s!", Log.DETAIL);
				return 0;
			}
			
			// Remove existing bank towns.
			// - Create list of existing banks.
			local bank_industry_list = List(GSIndustryList());
			bank_industry_list.Valuate(GSIndustry.GetIndustryType);
			bank_industry_list.KeepValue(industry_type_id);
			
			// - Convert to list of towns and remove from town list.
			bank_industry_list.Valuate(function(industry_id) { return GSTile.GetTownAuthority(GSIndustry.GetLocation(industry_id)) } );
			bank_industry_list.SetItemAsValue();
			bank_town_list.RemoveList(bank_industry_list);
			
			// Check if list is empty.
			if(bank_town_list.IsEmpty()) {
				Log.Warning(" - All valid towns have " + this.special_industry_names[0] + "s!", Log.DETAIL);
				return 0;
			}
			
			// Check to prospect.
			if(GSController.GetSetting("abnormal_industries_prospect") == 1) {
				
				// Try prospect
				if(GSIndustryType.ProspectIndustry(industry_type_id))
					return 1;	
				
				// Error handler.
				this.ErrorHandler(GSError.GetLastError());
				
				return 0;			
				
			// Else custom method.	
			} else {
			
				// Loop through town list.
				// Note due to the previous population valuate, this will now go by highest population.
				for(local town_id = bank_town_list.Begin(); !bank_town_list.IsEnd(); town_id = bank_town_list.Next()) {
					
					// Get randomised town tile list.
					local current_bank_town_list = this.GetTownHouseList(town_id);
					current_bank_town_list.Valuate(GSBase.RandItem);
					current_bank_town_list.Sort(GSList.SORT_BY_VALUE, GSList.SORT_ASCENDING);
					
					// Loop through town tile list.
					for(local tile_id = current_bank_town_list.Begin(); !current_bank_town_list.IsEnd(); tile_id = current_bank_town_list.Next()) {
						
						// Try build.
						if(GSIndustryType.BuildIndustry(industry_type_id, tile_id))
							return 1;
					}
					// Here means all tiles have been tried for this town.
				}
				// Here means all towns have been tried for this town list.
				
				// Error handler.
				this.ErrorHandler(GSError.GetLastError());
				
				return 0;
			}
		break;
		
		// "Oil Rig".
		case this.special_industry_names[1]:
		
			// Check if current date is before setting.
			if(GSDate.GetCurrentDate() < GSDate.GetDate(GSController.GetSetting("special_oil_rig"), 1, 1)) {
				Log.Warning(" - Year is less than " + GSController.GetSetting("special_oil_rig") + " for " + this.special_industry_names[1] + "s!", Log.DETAIL);
				
				return 0;
			}
			
			// Try prospect
			if(GSIndustryType.ProspectIndustry(industry_type_id))
				return 1;	
			
			// Error handler.
			this.ErrorHandler(GSError.GetLastError());
			
			return 0;
		break;
		
		// "Water Tower".
		case this.special_industry_names[2]:
		
			// Check towns with population > setting.
			// - Create town list.
			local watertower_town_list = GSTownList();
			// - Remove not in desert.
			watertower_town_list.Valuate(SuperLib.Town.IsDesertTown);
			watertower_town_list.KeepValue(1);
			// - Remove below population setting.
			watertower_town_list.Valuate(GSTown.GetPopulation);
			watertower_town_list.RemoveBelowValue(GSController.GetSetting("special_water_tower"));
			
			// Check if list is empty.
			if(watertower_town_list.IsEmpty()) {
				Log.Warning(" - No desert towns with more than " + GSController.GetSetting("special_water_tower") + " population, for " + this.special_industry_names[2] + "s!", Log.DETAIL);
				return 0;
			}
			
			// Remove existing water tower towns.
			// - Create list of existing watertowers.
			local watertower_industry_list = List(GSIndustryList());
			watertower_industry_list.Valuate(GSIndustry.GetIndustryType);
			watertower_industry_list.KeepValue(industry_type_id);
			
			// - Convert to list of towns and remove from town list.
			watertower_industry_list.Valuate(function(industry_id) { return GSTile.GetTownAuthority(GSIndustry.GetLocation(industry_id)) } );
			watertower_industry_list.SetItemAsValue();
			watertower_town_list.RemoveList(watertower_industry_list);
			
			// Check if list is empty.
			if(watertower_town_list.IsEmpty()) {
				Log.Warning(" - All valid desert towns have " + this.special_industry_names[2] + "s!", Log.DETAIL);
				return 0;
			}
			
			// Check to prospect.
			if(GSController.GetSetting("abnormal_industries_prospect") == 1) {
				
				// Try prospect
				if(GSIndustryType.ProspectIndustry(industry_type_id))
					return 1;	
				
				// Error handler.
				this.ErrorHandler(GSError.GetLastError());
				
				return 0;			
				
			// Else custom method.	
			} else {
			
				// Loop through town list.
				// Note due to the previous population valuate, this will now go by highest population.
				for(local town_id = watertower_town_list.Begin(); !watertower_town_list.IsEnd(); town_id = watertower_town_list.Next()) {
					
					// Get randomised town tile list.
					local current_watertower_town_list = this.GetTownHouseList(town_id);
					current_watertower_town_list.Valuate(GSBase.RandItem);
					current_watertower_town_list.Sort(GSList.SORT_BY_VALUE, GSList.SORT_ASCENDING);
					
					// Loop through town tile list.
					for(local tile_id = current_watertower_town_list.Begin(); !current_watertower_town_list.IsEnd(); tile_id = current_watertower_town_list.Next()) {
						
						// Try build.
						if(GSIndustryType.BuildIndustry(industry_type_id, tile_id))
							return 1;
					}
					// Here means all tiles have been tried for this town.
				}
				// Here means all towns have been tried for this list.
				
				// Error handler.
				this.ErrorHandler(GSError.GetLastError());
				
				return 0;
			}
			
		break;
		
		// "Lumber Mill".
		case this.special_industry_names[3]:
		
			// Check if must not build setting.
			if(GSController.GetSetting("special_lumber_mill") == 0) {
				Log.Warning(" - Not built as " + this.special_industry_names[3] + "s have been disabled!", Log.DETAIL);
				
				return 0;
			}
			
			// Try prospect
			if(GSIndustryType.ProspectIndustry(industry_type_id))
				return 1;	
			
			// Error handler.
			this.ErrorHandler(GSError.GetLastError());
			
			return 0;
		break;
		
		// Error.
		default:
			throw("MapConstructor.IndustrySpecialBuildMethod: Industry " + GSIndustryType.GetName(industry_type_id) + " not supported!");
	}
}

/**
 *	Test if the industry name is listed as abnormal for build methods, and get the unique location constraints.
 *	@param industry_name The industry name to test.
 *	@return An array of [distance_from_edge, prefer_terrain] if abnormal and null if not.
 *		distance_from_edge = null		Maximum distance from edge.
 *		local prefer_terrain = null		Only build able in this terrain.
 */
function MapConstructor::IndustryAbnormalCheck(industry_name) {
	
	local abnormal = false;
	local abnormal_array = [null, null];
	
	// - Oil Refinery.
	if(industry_name == this.abnormal_industry_names[0]) {
		
		// Assign abnormal.
		abnormal = true;
		abnormal_array[0] = GSGameSettings.GetValue("oil_refinery_limit");
	}
	// - Farm.
	else if(industry_name == this.abnormal_industry_names[1]) {
	
		// Check climate
		if(GSGame.GetLandscape () == GSGame.LT_ARCTIC) {
		
			// Assign abnormal.
			abnormal = true;
			abnormal_array[1] = GSTile.TERRAIN_NORMAL;
		}
	}
	// - Forest.
	else if(industry_name == this.abnormal_industry_names[2]) {
	
		// Check climate
		if(GSGame.GetLandscape () == GSGame.LT_ARCTIC) {
		
			// Assign abnormal.
			abnormal = true;
			abnormal_array[1] = GSTile.TERRAIN_SNOW;
		}
	}
	// - Water Supply.
	else if(industry_name == this.abnormal_industry_names[3]) {
		
		// Assign abnormal.
		abnormal = true;
		abnormal_array[1] = GSTile.TERRAIN_DESERT;
	}
	// - Fruit Plantation.
	else if(industry_name == this.abnormal_industry_names[4]) {
		
		// Assign abnormal.
		abnormal = true;
		abnormal_array[1] = GSTile.TERRAIN_NORMAL;
	}
	// - Rubber Plantation.
	else if(industry_name == this.abnormal_industry_names[5]) {
		
		// Assign abnormal.
		abnormal = true;
		abnormal_array[1] = GSTile.TERRAIN_NORMAL;
	}
	
	return abnormal ? abnormal_array : null;
}

/**
 *	Clear built game events so they are not mishandled by normal handler.
 *	@note Has to be called before HandleEvents() from the Main() function.
 */
 // TODO: Store non relevant events?
function MapConstructor::ClearEvents() {
	
	// Display msg
	Log.Info("+==============================+", Log.MAIN);
	Log.Info("Event clearing...", Log.MAIN);
	
	// Loop while events are waiting.
	while (GSEventController.IsEventWaiting()) {
	
		// Next event in variable.
  		local next_event = GSEventController.GetNextEvent();
		
	}
}

/**
 *	Detect and manage game events.
 *	@note Has to be called from the Main() function.
 */
 // TODO: demolish random code built industries...requires change in Precondition: Valid GSCompanyMode active in scope...for GSTile.DemolishTile().
function MapConstructor::HandleEvents() {
	
	// Display msg
	Log.Info("+==============================+", Log.MAIN);
	Log.Info("Event handling...", Log.MAIN);
	
	// Loop while events are waiting.
	while (GSEventController.IsEventWaiting()) {
	
		// Next event in variable.
  		local next_event = GSEventController.GetNextEvent();
		
		// Switch for event type.
  		switch (next_event.GetEventType()) {
		
  			// Event: New industry
		    case GSEvent.ET_INDUSTRY_OPEN:
		    	// Display status msg.
		    	Log.Info("> Detected 'new industry opened' event.", Log.SUB);
		    	// Convert the event.
		      	local event_controler = GSEventIndustryOpen.Convert(next_event);
		      	// Get the industry ID.
		      	local industry_id  = event_controler.GetIndustryID();
		    	Log.Warning("> Nothing can be done about this in the current API!!!", Log.SUB);
				/*
		      	// Get the tile of the industry.
		      	local tile_id = GSIndustry.GetLocation(industry_id);
		      	// Demolish the industry and check for error.
		      	if(!GSTile.DemolishTile(tile_id))
					this.ErrorHandler(GSError.GetLastError());
				*/
		    break;
			
  			// Event: Industry closed.
		    case GSEvent.ET_INDUSTRY_CLOSE:
		    	// Display status msg.
		    	Log.Info("> Detected 'industry closed' event. This is accommodated by the next build refresh.", Log.SUB)
		    	// Convert the event.
		      	local event_controler = GSEventIndustryClose.Convert(next_event);
		      	// Get the industry ID.
		      	local industry_id  = event_controler.GetIndustryID();
		      	// Do nothing, as reduced number of industries will be handled in the next refresh loop.
		    break;
			
  			// Event: Town founded.
		    case GSEvent.ET_TOWN_FOUNDED:
		    	// Display status msg.
		    	Log.Info("> Detected 'town founded' event..", Log.SUB)
		    	// Convert the event.
		      	local event_controler = GSEventTownFounded.Convert(next_event);
		      	// Get the town ID.
		      	local town_id  = event_controler.GetTownID();
		    	Log.Warning("> Nothing can be done about this in the current API!!!", Log.SUB);
		    break;
  		}
	}
}

/**
 *	Display a msg to all players in the form of a message dialogue.
 *	@param text A string of the text to display.
 *	@note Will end initial tick run if happening currently!
 */
function MapConstructor::WarningBox(text) {

	// Validate
	if(typeof text != "string")
		throw("MapConstructor::WarningBox: Param is not a string!");
	
	// Ensure game start.
	GSController.Sleep(1);
	
	// Close previous windows.
	GSGoal.CloseQuestion(1);
	
	// Create window.
	GSGoal.Question(1, GSCompany.COMPANY_INVALID, text, GSGoal.QT_INFORMATION, GSGoal.BUTTON_CLOSE);
	
	return;
}

/**
 *	Generic error handler which displays the latest error in log and breaks.
 *	For errors to be detected GetLastError must be called literally as the next thing
 *		after a DO COMMAND, no variables assignments in squirrel, nothing! Eg:
 *			if(!DoCommand(param))
 *				// Error handler.
 *				this.ErrorHandler(GSError.GetLastError());
 *	@param error Call GSError.GetLastError() and pass the variable in as parameter.
 */
function MapConstructor::ErrorHandler(error) {

	// If no error then return.
	if(error == GSError.ERR_NONE) 
		return;

	Log.Error("", Log.MAIN);
	Log.Error("Error detected!", Log.MAIN);
	
	// Get error category and string.
	local error_category = GSError.GetErrorCategory();
	local error_string = GSError.GetLastErrorString();
	
	Log.Error("Error code: " + error_string, Log.MAIN);
	
	// Switch for error category.
	switch(error_category) {
		
		case GSError.ERR_CAT_NONE: Log.Error("Error messages not related to any category!", Log.MAIN);
		break;
		case GSError.ERR_CAT_GENERAL: Log.Error("Error messages related to general things!", Log.MAIN);
		break;
		case GSError.ERR_CAT_BRIDGE: Log.Error("Error messages related to building / removing bridges!", Log.MAIN);
		break;
		case GSError.ERR_CAT_MARINE: Log.Error("Error messages related to building / removing ships, docks and channels!", Log.MAIN);
		break;
		case GSError.ERR_CAT_ORDER: Log.Error("Error messages related to managing orders!", Log.MAIN);
		break;
		case GSError.ERR_CAT_RAIL: Log.Error("Error messages related to building / maintaining rails!", Log.MAIN);
		break;
		case GSError.ERR_CAT_ROAD: Log.Error("Error messages related to building / maintaining roads!", Log.MAIN);
		break;
		case GSError.ERR_CAT_SIGN: Log.Error("Error messages related to building / removing signs!", Log.MAIN);
		break;
		case GSError.ERR_CAT_STATION: Log.Error("Error messages related to building / maintaining stations!", Log.MAIN);
		break;
		case GSError.ERR_CAT_TILE: Log.Error("Error messages related to raising / lowering and demolishing tiles!", Log.MAIN);
		break;
		case GSError.ERR_CAT_TUNNEL: Log.Error("Error messages related to building / removing tunnels!", Log.MAIN);
		break;
		case GSError.ERR_CAT_VEHICLE: Log.Error("Error messages related to building / maintaining vehicles!", Log.MAIN);
		break;
		case GSError.ERR_CAT_WAYPOINT: Log.Error("Error messages related to building / maintaining waypoints!", Log.MAIN);
		break;
		default: Log.Error("MapConstructor::ErrorHandler: Unhandled error category!", Log.MAIN);
	}

	switch(error) {
		// GENERAL - INTERNAL.
		case GSError.ERR_UNKNOWN: Log.Error("If an error occurred and the error wasn't mapped!", Log.SUB);
		break;
		case GSError.ERR_PRECONDITION_FAILED: Log.Error("A precondition is not met!", Log.SUB);
		break;
		case GSError.ERR_PRECONDITION_STRING_TOO_LONG: Log.Error("A string supplied was too long!", Log.SUB);
		break;
		case GSError.ERR_PRECONDITION_TOO_MANY_PARAMETERS: Log.Error("A string had too many parameters!", Log.SUB);
		break;
		case GSError.ERR_PRECONDITION_INVALID_COMPANY: Log.Error("The company you use is invalid!", Log.SUB);
		break;
		case GSError.ERR_NEWGRF_SUPPLIED_ERROR: Log.Error("An error returned by a NewGRF!", Log.SUB);
		break;
		// GENERAL - GAME.
		case GSError.ERR_NOT_ENOUGH_CASH: Log.Error("Not enough cash to perform the previous action!", Log.SUB);
		break;
		case GSError.ERR_LOCAL_AUTHORITY_REFUSES: Log.Error("Local authority won't allow the previous action!", Log.SUB);
		break;
		case GSError.ERR_ALREADY_BUILT: Log.Error("The piece of infrastructure you tried to build is already in place!", Log.SUB);
		break;
		case GSError.ERR_AREA_NOT_CLEAR: Log.Error("Area isn't clear, try to demolish the building on it!", Log.SUB);
		break;
		case GSError.ERR_OWNED_BY_ANOTHER_COMPANY: Log.Error("Area / property is owned by another company!", Log.SUB);
		break;
		case GSError.ERR_NAME_IS_NOT_UNIQUE: Log.Error("The name given is not unique for the object type!", Log.SUB);
		break;
		case GSError.ERR_FLAT_LAND_REQUIRED: Log.Error("The building you want to build requires flat land!", Log.SUB);
		break;
		case GSError.ERR_LAND_SLOPED_WRONG: Log.Error("Land is sloped in the wrong direction for this build action!", Log.SUB);
		break;
		case GSError.ERR_VEHICLE_IN_THE_WAY: Log.Error("A vehicle is in the way!", Log.SUB);
		break;
		case GSError.ERR_SITE_UNSUITABLE: Log.Error("Site is unsuitable!", Log.SUB);
		break;
		case GSError.ERR_TOO_CLOSE_TO_EDGE: Log.Error("Too close to the edge of the map!", Log.SUB);
		break;
		case GSError.ERR_STATION_TOO_SPREAD_OUT: Log.Error("Station is too spread out!", Log.SUB);
		break;
		// BRIDGE.
		case GSBridge.ERR_BRIDGE_TYPE_UNAVAILABLE: Log.Error("The bridge you want to build is not available yet, or it is not available for the requested length!", Log.SUB);
		break;
		case GSBridge.ERR_BRIDGE_CANNOT_END_IN_WATER: Log.Error("One (or more) of the bridge head(s) ends in water!", Log.SUB);
		break;
		case GSBridge.ERR_BRIDGE_HEADS_NOT_ON_SAME_HEIGHT: Log.Error("The bride heads need to be on the same height!", Log.SUB);
		// MARINE.
		case GSMarine.ERR_MARINE_MUST_BE_BUILT_ON_WATER: Log.Error("Infrastructure must be built on water!", Log.SUB);
		// ORDER.
		case GSOrder.ERR_ORDER_TOO_MANY: Log.Error("No more space for orders!", Log.SUB);
		break;
		case GSOrder.ERR_ORDER_TOO_FAR_AWAY_FROM_PREVIOUS_DESTINATION: Log.Error("Destination of new order is to far away from the previous order!", Log.SUB);
		break;
		case GSOrder.ERR_ORDER_AIRCRAFT_NOT_ENOUGH_RANGE: Log.Error("Aircraft has not enough range to copy/share orders!", Log.SUB);
		break;
		// RAIL.
		case GSRail.ERR_CROSSING_ON_ONEWAY_ROAD: Log.Error("One-way roads cannot have crossings!", Log.SUB);
		break;
		case GSRail.ERR_UNSUITABLE_TRACK: Log.Error("No suitable track could be found!", Log.SUB);
		break;
		case GSRail.ERR_RAILTYPE_DISALLOWS_CROSSING: Log.Error("This railtype cannot have crossings!", Log.SUB);
		break;
		// ROAD.
		case GSRoad.ERR_ROAD_WORKS_IN_PROGRESS: Log.Error("Road works are in progress!", Log.SUB);
		break;
		case GSRoad.ERR_ROAD_DRIVE_THROUGH_WRONG_DIRECTION: Log.Error("Drive through is in the wrong direction!", Log.SUB);
		break;
		case GSRoad.ERR_ROAD_CANNOT_BUILD_ON_TOWN_ROAD: Log.Error("Drive through roads can't be build on town owned roads!", Log.SUB);
		break;
		case GSRoad.ERR_ROAD_ONE_WAY_ROADS_CANNOT_HAVE_JUNCTIONS: Log.Error("One way roads can't have junctions!", Log.SUB);
		break;
		// SIGN.
		case GSSign.ERR_SIGN_TOO_MANY_SIGNS: Log.Error("Too many signs have been placed!", Log.SUB);
		break;
		// STATION.
		case GSStation.ERR_STATION_TOO_CLOSE_TO_ANOTHER_STATION: Log.Error("The station is build too close to another station, airport or dock!", Log.SUB);
		break;
		case GSStation.ERR_STATION_TOO_MANY_STATIONS: Log.Error("There are too many stations, airports and docks in the game!", Log.SUB);
		break;
		case GSStation.ERR_STATION_TOO_MANY_STATIONS_IN_TOWN: Log.Error("There are too many stations, airports of docks in a town!", Log.SUB);
		break;
		// TILE.
		case GSTile.ERR_TILE_TOO_HIGH: Log.Error("Tile can't be raised any higher!", Log.SUB);
		break;
		case GSTile.ERR_TILE_TOO_LOW: Log.Error("Tile can't be lowered any lower!", Log.SUB);
		break;
		case GSTile.ERR_AREA_ALREADY_FLAT: Log.Error("The area was already flat!", Log.SUB);
		break;
		case GSTile.ERR_EXCAVATION_WOULD_DAMAGE: Log.Error("There is a tunnel underneath!", Log.SUB);
		break;
		// TUNNEL.
		case GSTunnel.ERR_TUNNEL_CANNOT_BUILD_ON_WATER: Log.Error("Can't build tunnels on water!", Log.SUB);
		break;
		case GSTunnel.ERR_TUNNEL_START_SITE_UNSUITABLE: Log.Error("The start tile must slope either North, South, West or East!", Log.SUB);
		break;
		case GSTunnel.ERR_TUNNEL_ANOTHER_TUNNEL_IN_THE_WAY: Log.Error("Another tunnel is in the way!", Log.SUB);
		break;
		case GSTunnel.ERR_TUNNEL_END_SITE_UNSUITABLE: Log.Error("Unable to excavate land at the end to create the tunnel's exit!", Log.SUB);
		break;
		// VEHICLE.
		case GSVehicle.ERR_VEHICLE_TOO_MANY: Log.Error("Too many vehicles in the game, can't build any more!", Log.SUB);
		break;
		case GSVehicle.ERR_VEHICLE_NOT_AVAILABLE: Log.Error("Vehicle is not available!", Log.SUB);
		break;
		case GSVehicle.ERR_VEHICLE_BUILD_DISABLED: Log.Error("Vehicle can't be build due to game settings!", Log.SUB);
		break;
		case GSVehicle.ERR_VEHICLE_WRONG_DEPOT: Log.Error("Vehicle can't be build in the selected depot!", Log.SUB);
		break;
		case GSVehicle.ERR_VEHICLE_CANNOT_SEND_TO_DEPOT: Log.Error("Vehicle can't return to the depot!", Log.SUB);
		break;
		case GSVehicle.ERR_VEHICLE_CANNOT_START_STOP: Log.Error("Vehicle can't start / stop!", Log.SUB);
		break;
		case GSVehicle.ERR_VEHICLE_CANNOT_TURN: Log.Error("Vehicle can't turn!", Log.SUB);
		break;
		case GSVehicle.ERR_VEHICLE_CANNOT_REFIT: Log.Error("Vehicle can't be refit!", Log.SUB);
		break;
		case GSVehicle.ERR_VEHICLE_IS_DESTROYED: Log.Error("Vehicle is destroyed!", Log.SUB);
		break;
		case GSVehicle.ERR_VEHICLE_NOT_IN_DEPOT: Log.Error("Vehicle is not in a depot!", Log.SUB);
		break;
		case GSVehicle.ERR_VEHICLE_IN_FLIGHT: Log.Error("Vehicle is flying!", Log.SUB);
		break;
		case GSVehicle.ERR_VEHICLE_NO_POWER: Log.Error("Vehicle is without power!", Log.SUB);
		break;
		case GSVehicle.ERR_VEHICLE_TOO_LONG: Log.Error("Vehicle would get too long during construction!", Log.SUB);
		break;
		// WAYPOINT.
		case GSWaypoint.ERR_WAYPOINT_TOO_CLOSE_TO_ANOTHER_WAYPOINT: Log.Error("The waypoint is build too close to another waypoint!", Log.SUB);
		break;
		case GSWaypoint.ERR_WAYPOINT_ADJOINS_MULTIPLE_WAYPOINTS: Log.Error("The waypoint would join more than one existing waypoint together!", Log.SUB);
		break;
		default: Log.Error("MapConstructor::ErrorHandler: Unhandled error!", Log.MAIN);
	}

	// Break
	Log.Break("MapConstructor::ErrorHandler");
	Log.Error("", Log.MAIN);
	
	return;
}

/**
 *	Get a random tile on the map.
 *	@return The tile id.
 */
function MapConstructor::GetRandomTile() {
	return GSMap.GetTileIndex(GSBase.RandRange(this.map_size_x), GSBase.RandRange(this.map_size_y));
}

/**
 *	Get a random tile on the map close to edge.
 *	@param range The maximum distance from edge.
 *	@return The tile id.
 */
function MapConstructor::GetRandomTileCloseToEdge(range) {
	
	local x_coord = null;
	local y_coord = null;
	
	if(GSBase.Chance(1, 2)) {
		
		x_coord = GSBase.Chance(1, 2) ? (0 + GSBase.RandRange(range + 1)) : (this.map_size_x - range + GSBase.RandRange(range + 1));
		y_coord = GSBase.RandRange(this.map_size_y);
		
	} else {
		
		x_coord = GSBase.RandRange(this.map_size_x);
		y_coord = GSBase.Chance(1, 2) ? (0 + GSBase.RandRange(range + 1)) : (this.map_size_y - range + GSBase.RandRange(range + 1));
		
	}
	
	return GSMap.GetTileIndex(x_coord, y_coord);
}

/**
 *	Get a random tile on the map close to given tile, clamped to edges.
 *	@param tile_index The tile to be close to.
 *	@param max_radius The maximum distance.
 *	@param min_radius The minimum distance.
 *	@pre max_radius > min_radius && min_radius > 0 && max_radius > 0.
 *	@return The tile id.
 */
function MapConstructor::GetRandomTileCloseToTile(tile_index, max_radius, min_radius = 0) {
	
	local x_rand = 0;
	local y_rand = 0;
	
	GSBase.Chance(1, 2) ? x_rand = min_radius : y_rand = min_radius;
	
	local x_coord = null;
	local y_coord = null;
	local tile = null;
	
	do {
	
		x_coord = max(0, min(this.map_size_x, GSMap.GetTileX(tile_index) + ((x_rand + GSBase.RandRange(max_radius - x_rand)) * (GSBase.Chance(1, 2) ? 1 : -1))));
		y_coord = max(0, min(this.map_size_y, GSMap.GetTileY(tile_index) + ((y_rand + GSBase.RandRange(max_radius - y_rand)) * (GSBase.Chance(1, 2) ? 1 : -1))));
		tile = GSMap.GetTileIndex(x_coord, y_coord);
	
	} while(GSMap.DistanceMax(tile_index, tile) > max_radius);
	
	return tile;
}

/**
 *	Get the closest industry tile.
 *	@param tile_index The tile to search from.
 *	@return The industry ID of the closest industry or null if there are no industries on the map.
 */
function MapConstructor::GetClosestIndustry(tile_index) {

	// Create a list of all industries.
	local industry_list = GSIndustryList();
	
	// If count is 0, return null.
	if(industry_list.IsEmpty())
		return null;
	
	// Valuate by distance from tile.
	industry_list.Valuate(GSIndustry.GetDistanceManhattanToTile, tile_index);
	
	// Sort smallest to largest.
	industry_list.Sort(GSList.SORT_BY_VALUE, GSList.SORT_ASCENDING);
	
	// Return the top one.
	return industry_list.Begin();
}

/**
 *	Quick and dirty way to get a tile lists of town houses.
 * 		Get a large rectangle of tiles around a town, meant to include 
 *		every house and authority tile. Filter with AITile.GetTownAuthority 
 * 		to refine the area shape around nearby towns.
 * 		Uses a superlib tile function.
 *	@param town_index The town to get tiles from.
 *	@return An GSTileList of tiles with approximate town size rectangle.
 */
function MapConstructor::GetTownTileList(town_index) {

	/* 
	 *	GSTile.GetTownAuthority():
	 *		A large "square" around town, non overlapping, of tiles affected by town. 
	 *		The "Local Authority" from ? tooltip.
	 *	GSTile.IsWithinTownInfluence():
	 *		A "circle" of tiles around town center, affected by a setting.
	 *		Building a tree will influence the rating of the town returned by 
	 *		GSTile::GetClosestTown(tile) if it's closer then settings_game.economy.dist_local_authority 
	 *		(you can get that value via AIGameSettings.GetValue("economy.dist_local_authority"); )
	 */

	local house_tile_factor = 0.90;					///< A float multiplier, larger means a bigger square of town tiles, smaller means smaller list.
	 
	local town_radius = (GSTown.GetHouseCount(town_index) * house_tile_factor).tointeger();
	return SuperLib.Tile.MakeTileRectAroundTile(GSTown.GetLocation(town_index), town_radius);
}

/**
 *	Get a tile lists of town building tiles in this town.
 * 	@param town_index The town to get tiles from.
 *	@return Returns an GSTileList of tiles.
 */
 // TODO: remove dependancy on passenger cargo...requires change in Precondition: Valid GSCompanyMode active in scope...for GSTile.DemolishTile().
function MapConstructor::GetTownHouseList(town_index) {

	// Get town tiles.
	local house_count = GSTown.GetHouseCount(town_index);				//* ~1.25 to account for multi tile buildings
	local town_tile_list = List(this.GetTownTileList(town_index));
	
	// Filter for houses.
	town_tile_list.Valuate(GSTile.GetTownAuthority);
	town_tile_list.KeepValue(town_index);
	town_tile_list.Valuate(GSTile.GetOwner);
	town_tile_list.KeepValue(-1);
	town_tile_list.Valuate(GSIndustry.GetIndustryID);
	town_tile_list.KeepValue(65535);
	town_tile_list.Valuate(GSTile.IsBuildable);
	town_tile_list.KeepValue(0);
	town_tile_list.Valuate(GSTile.GetCargoAcceptance, this.cargo_paxid, 1, 1, 0);
	town_tile_list.KeepAboveValue(0);
	
	// Check if we got all the houses.
	if(town_tile_list.Count() >= house_count) {
	
		return town_tile_list;
	
	// Else if we found some tiles, debug.
	} else if(town_tile_list.Count() > 0) {
	
		// Error handler.
		this.ErrorHandler(GSError.GetLastError());
		
		// Debug
		foreach(town_tile, _ in town_tile_list)
			GSSign.BuildSign(town_tile, "Town building");
		
		throw("MapConstructor::GetTownHouseList: Found less than required town tile list!");
		
		return null;
		
	// Else serious error.
	} else {
	
		// Error handler.
		Log.Error(GSError.GetLastErrorString());
		
		throw("MapConstructor::GetNewTownHouseList: Empty list!");
	}
}
